r"""
Graded Algebras
"""
#*****************************************************************************
#  Copyright (C) 2008      Teresa Gomez-Diaz (CNRS) <Teresa.Gomez-Diaz@univ-mlv.fr>
#                2008-2011 Nicolas M. Thiery <nthiery at users.sf.net>
#
#  Distributed under the terms of the GNU General Public License (GPL)
#                  http://www.gnu.org/licenses/
#******************************************************************************

from sage.categories.graded_modules import GradedModulesCategory
from sage.categories.signed_tensor import SignedTensorProductsCategory
from sage.misc.cachefunc import cached_method

class GradedAlgebras(GradedModulesCategory):
    """
    The category of graded algebras

    EXAMPLES::

        sage: GradedAlgebras(ZZ)
        Category of graded algebras over Integer Ring
        sage: GradedAlgebras(ZZ).super_categories()
        [Category of filtered algebras over Integer Ring,
         Category of graded modules over Integer Ring]

    TESTS::

        sage: TestSuite(GradedAlgebras(ZZ)).run()
    """
    class ParentMethods:
        def graded_algebra(self):
            """
            Return the associated graded algebra to ``self``.

            Since ``self`` is already graded, this just returns
            ``self``.

            EXAMPLES::

                sage: m = SymmetricFunctions(QQ).m()
                sage: m.graded_algebra() is m
                True
            """
            return self

    class ElementMethods:
        pass

    class SubcategoryMethods:
        def SignedTensorProducts(self):
            r"""
            Return the full subcategory of objects of ``self`` constructed
            as signed tensor products.

            .. SEEALSO::

                - :class:`~sage.categories.signed_tensor.SignedTensorProductsCategory`
                - :class:`~.covariant_functorial_construction.CovariantFunctorialConstruction`

            EXAMPLES::

                sage: AlgebrasWithBasis(QQ).Graded().SignedTensorProducts()
                Category of signed tensor products of graded algebras with basis
                 over Rational Field
            """
            return SignedTensorProductsCategory.category_of(self)

    class SignedTensorProducts(SignedTensorProductsCategory):
        @cached_method
        def extra_super_categories(self):
            """
            EXAMPLES::

                sage: Algebras(QQ).Graded().SignedTensorProducts().extra_super_categories()
                [Category of graded algebras over Rational Field]
                sage: Algebras(QQ).Graded().SignedTensorProducts().super_categories()
                [Category of graded algebras over Rational Field]

            Meaning: a signed tensor product of algebras is an algebra
            """
            return [self.base_category()]




r"""
Dynamic documentation for instances of classes

The functionality in this module allows to define specific docstrings
of *instances* of a class, which are different from the class docstring.
A typical use case is given by cached methods: the documentation of a
cached method should not be the documentation of the class
:class:`CachedMethod`; it should be the documentation of the underlying
method.

In order to use this, define a class docstring as usual. Also define a
method ``def _instancedoc_(self)`` which should return the docstring of
the instance ``self``. Finally, add the decorator ``@instancedoc`` to
the class.

.. WARNING::

    Since the ``__doc__`` attribute is never inherited, the decorator
    ``@instancedoc`` must be added to all subclasses of the class
    defining ``_instancedoc_``. Doing it on the base class is not
    sufficient.

EXAMPLES::

    sage: from sage.docs.instancedoc import instancedoc
    sage: @instancedoc
    ....: class X(object):
    ....:     "Class docstring"
    ....:     def _instancedoc_(self):
    ....:         return "Instance docstring"
    sage: X.__doc__
    'Class docstring'
    sage: X().__doc__
    'Instance docstring'

For a Cython ``cdef class``, a decorator cannot be used. Instead, call
:func:`instancedoc` as a function after defining the class::

    sage: cython('''
    ....: from sage.docs.instancedoc import instancedoc
    ....: cdef class Y:
    ....:     "Class docstring"
    ....:     def _instancedoc_(self):
    ....:         return "Instance docstring"
    ....: instancedoc(Y)
    ....: ''')
    sage: Y.__doc__
    'File:...\nClass docstring'
    sage: Y().__doc__
    'Instance docstring'

One can still add a custom ``__doc__`` attribute on a particular
instance::

    sage: obj = X()
    sage: obj.__doc__ = "Very special doc"
    sage: print(obj.__doc__)
    Very special doc

This normally does not work on extension types::

    sage: Y().__doc__ = "Very special doc"
    Traceback (most recent call last):
    ...
    AttributeError: attribute '__doc__' of 'Y' objects is not writable

This is an example involving a metaclass, where the instances are
classes. In this case, the ``_instancedoc_`` from the metaclass is only
used if the instance of the metaclass (the class) does not have a
docstring::

    sage: @instancedoc
    ....: class Meta(type):
    ....:     "Metaclass doc"
    ....:     def _instancedoc_(self):
    ....:         return "Docstring for {}".format(self)
    sage: class T(metaclass=Meta):
    ....:     pass
    sage: print(T.__doc__)
    Docstring for <class '__main__.T'>
    sage: class U(metaclass=Meta):
    ....:     "Special doc for U"
    sage: print(U.__doc__)
    Special doc for U

TESTS:

Check that inheritance works (after passing the subclass to
:func:`instancedoc`)::

    sage: @instancedoc
    ....: class A(object):
    ....:     "Class A docstring"
    ....:     def _instancedoc_(self):
    ....:         return "Instance docstring"
    sage: class B(A):
    ....:     "Class B docstring"
    sage: B.__doc__
    'Class B docstring'
    sage: B().__doc__  # Ideally, this would return the instance docstring
    'Class B docstring'
    sage: B = instancedoc(B)
    sage: B.__doc__
    'Class B docstring'
    sage: B().__doc__
    'Instance docstring'
"""

#*****************************************************************************
#       Copyright (C) 2017 Jeroen Demeyer <J.Demeyer@UGent.be>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 2 of the License, or
# (at your option) any later version.
#                  https://www.gnu.org/licenses/
#*****************************************************************************

from cpython.object cimport PyObject, PyTypeObject

cdef extern from *:
    cdef int PyDict_SetItemString(PyObject*, const char*, object) except -1
    cdef void PyType_Modified(PyTypeObject*)

cdef inline PyTypeObject* TypeObject(cls) except NULL:
    if not isinstance(cls, type):
        raise TypeError(f"expected type, got {cls!r}")
    return <PyTypeObject*>cls


cdef class InstanceDocDescriptor:
    """
    Descriptor for dynamic documentation, to be installed as the
    ``__doc__`` attribute.

    INPUT:

    - ``classdoc`` -- (string) class documentation

    - ``instancedoc`` -- (method) documentation for an instance

    - ``attr`` -- (string, default ``__doc__``) attribute name to use
      for custom docstring on the instance.

    EXAMPLES::

        sage: from sage.docs.instancedoc import InstanceDocDescriptor
        sage: def instancedoc(self):
        ....:     return "Instance doc"
        sage: docattr = InstanceDocDescriptor("Class doc", instancedoc)
        sage: class Z(object):
        ....:     __doc__ = InstanceDocDescriptor("Class doc", instancedoc)
        sage: Z.__doc__
        'Class doc'
        sage: Z().__doc__
        'Instance doc'

    We can still override the ``__doc__`` attribute of the instance::

        sage: obj = Z()
        sage: obj.__doc__ = "Custom doc"
        sage: obj.__doc__
        'Custom doc'
        sage: del obj.__doc__
        sage: obj.__doc__
        'Instance doc'
    """
    cdef classdoc
    cdef instancedoc
    cdef attr

    def __init__(self, classdoc, instancedoc, attr="__doc__"):
        """
        TESTS::

            sage: from sage.docs.instancedoc import InstanceDocDescriptor
            sage: InstanceDocDescriptor(None, None)
            <sage.docs.instancedoc.InstanceDocDescriptor object at ...>
        """
        self.classdoc = classdoc
        self.instancedoc = instancedoc
        self.attr = intern(attr)

    def __get__(self, obj, typ):
        """
        TESTS::

            sage: from sage.docs.instancedoc import InstanceDocDescriptor
            sage: def instancedoc(self):
            ....:     return "Doc for {!r}".format(self)
            sage: descr = InstanceDocDescriptor("Class doc", instancedoc)
            sage: descr.__get__(None, object)
            'Class doc'
            sage: descr.__get__(42, type(42))
            'Doc for 42'
        """
        if obj is None:
            return self.classdoc

        # First, try the attribute self.attr (typically __doc__)
        # on the instance
        try:
            objdict = obj.__dict__
        except AttributeError:
            pass
        else:
            doc = objdict.get(self.attr)
            if doc is not None:
                return doc

        return self.instancedoc(obj)

    def __set__(self, obj, value):
        """
        TESTS::

            sage: from sage.docs.instancedoc import InstanceDocDescriptor
            sage: def instancedoc(self):
            ....:     return "Doc for {!r}".format(self)
            sage: descr = InstanceDocDescriptor("Class doc", instancedoc)
            sage: class X(object): pass
            sage: obj = X()
            sage: descr.__set__(obj, "Custom doc")
            sage: obj.__doc__
            'Custom doc'

            sage: descr.__set__([], "Custom doc")
            Traceback (most recent call last):
            ...
            AttributeError: attribute '__doc__' of 'list' objects is not writable
            sage: descr.__set__(object, "Custom doc")
            Traceback (most recent call last):
            ...
            AttributeError: attribute '__doc__' of 'type' objects is not writable
        """
        try:
            obj.__dict__[self.attr] = value
        except (AttributeError, TypeError):
            raise AttributeError(f"attribute '{self.attr}' of '{type(obj).__name__}' objects is not writable")

    def __delete__(self, obj):
        """
        TESTS::

            sage: from sage.docs.instancedoc import InstanceDocDescriptor
            sage: def instancedoc(self):
            ....:     return "Doc for {!r}".format(self)
            sage: descr = InstanceDocDescriptor("Class doc", instancedoc)
            sage: class X(object): pass
            sage: obj = X()
            sage: obj.__doc__ = "Custom doc"
            sage: descr.__delete__(obj)
            sage: print(obj.__doc__)
            None
            sage: descr.__delete__(obj)
            Traceback (most recent call last):
            ...
            AttributeError: 'X' object has no attribute '__doc__'

            sage: descr.__delete__([])
            Traceback (most recent call last):
            ...
            AttributeError: attribute '__doc__' of 'list' objects is not writable
            sage: descr.__delete__(object)
            Traceback (most recent call last):
            ...
            AttributeError: attribute '__doc__' of 'type' objects is not writable
        """
        try:
            del obj.__dict__[self.attr]
        except (AttributeError, TypeError):
            raise AttributeError(f"attribute '{self.attr}' of '{type(obj).__name__}' objects is not writable")
        except KeyError:
            raise AttributeError(f"'{type(obj).__name__}' object has no attribute '{self.attr}'")


def instancedoc(cls):
    """
    Add support for ``_instancedoc_`` to the class ``cls``.

    Typically, this will be used as decorator.

    INPUT:

    - ``cls`` -- a new-style class

    OUTPUT: ``cls``

    .. WARNING::

        ``instancedoc`` mutates the given class. So you are *not* supposed
        to use it as ``newcls = instancedoc(cls)`` because that would
        mutate ``cls`` (and ``newcls`` would be the same object as ``cls``)

    TESTS:

    We get a useful error message if ``_instancedoc_`` is not defined::

        sage: from sage.docs.instancedoc import instancedoc
        sage: class X(object): pass
        sage: instancedoc(X)
        Traceback (most recent call last):
        ...
        TypeError: instancedoc requires <class '__main__.X'> to have an '_instancedoc_' attribute

    This does not work on old-style classes or things which are not a
    class at all::

        sage: instancedoc(7)
        Traceback (most recent call last):
        ...
        TypeError: expected type, got 7

        sage: class OldStyle: pass
        sage: instancedoc(OldStyle)
        Traceback (most recent call last):
        ...
        TypeError: instancedoc requires <class '__main__.OldStyle'> to have an '_instancedoc_' attribute
    """
    cdef PyTypeObject* tp = TypeObject(cls)
    try:
        instdoc = cls._instancedoc_
    except AttributeError:
        raise TypeError(f"instancedoc requires {cls!r} to have an '_instancedoc_' attribute")
    docattr = InstanceDocDescriptor(cls.__doc__, instdoc)
    PyDict_SetItemString(tp.tp_dict, "__doc__", docattr)
    tp.tp_doc = NULL
    PyType_Modified(tp)
    return cls



"""
Cartesian Product Functorial Construction

AUTHORS:

 - Nicolas M. Thiery (2008-2010): initial revision and refactorization
"""
#*****************************************************************************
#  Copyright (C) 2010 Nicolas M. Thiery <nthiery at users.sf.net>
#
#  Distributed under the terms of the GNU General Public License (GPL)
#                  http://www.gnu.org/licenses/
#*****************************************************************************

from sage.misc.lazy_import import lazy_import
from sage.categories.covariant_functorial_construction import CovariantFunctorialConstruction, CovariantConstructionCategory
from sage.categories.pushout import MultivariateConstructionFunctor

native_python_containers   = set([tuple, list, set, frozenset, range])

class CartesianProductFunctor(CovariantFunctorialConstruction, MultivariateConstructionFunctor):
    """
    The Cartesian product functor.

    EXAMPLES::

        sage: cartesian_product
        The cartesian_product functorial construction

    ``cartesian_product`` takes a finite collection of sets, and
    constructs the Cartesian product of those sets::

        sage: A = FiniteEnumeratedSet(['a','b','c'])
        sage: B = FiniteEnumeratedSet([1,2])
        sage: C = cartesian_product([A, B]); C
        The Cartesian product of ({'a', 'b', 'c'}, {1, 2})
        sage: C.an_element()
        ('a', 1)
        sage: C.list()         # todo: not implemented
        [['a', 1], ['a', 2], ['b', 1], ['b', 2], ['c', 1], ['c', 2]]

    If those sets are endowed with more structure, say they are
    monoids (hence in the category ``Monoids()``), then the result is
    automatically endowed with its natural monoid structure::

        sage: M = Monoids().example()
        sage: M
        An example of a monoid: the free monoid generated by ('a', 'b', 'c', 'd')
        sage: M.rename('M')
        sage: C = cartesian_product([M, ZZ, QQ])
        sage: C
        The Cartesian product of (M, Integer Ring, Rational Field)
        sage: C.an_element()
        ('abcd', 1, 1/2)
        sage: C.an_element()^2
        ('abcdabcd', 1, 1/4)
        sage: C.category()
        Category of Cartesian products of monoids

        sage: Monoids().CartesianProducts()
        Category of Cartesian products of monoids

    The Cartesian product functor is covariant: if ``A`` is a
    subcategory of ``B``, then ``A.CartesianProducts()`` is a
    subcategory of ``B.CartesianProducts()`` (see also
    :class:`~sage.categories.covariant_functorial_construction.CovariantFunctorialConstruction`)::

        sage: C.categories()
        [Category of Cartesian products of monoids,
         Category of monoids,
         Category of Cartesian products of semigroups,
         Category of semigroups,
         Category of Cartesian products of unital magmas,
         Category of Cartesian products of magmas,
         Category of unital magmas,
         Category of magmas,
         Category of Cartesian products of sets,
         Category of sets, ...]

        [Category of Cartesian products of monoids,
         Category of monoids,
         Category of Cartesian products of semigroups,
         Category of semigroups,
         Category of Cartesian products of magmas,
         Category of unital magmas,
         Category of magmas,
         Category of Cartesian products of sets,
         Category of sets,
         Category of sets with partial maps,
         Category of objects]

    Hence, the role of ``Monoids().CartesianProducts()`` is solely to
    provide mathematical information and algorithms which are relevant
    to Cartesian product of monoids. For example, it specifies that
    the result is again a monoid, and that its multiplicative unit is
    the Cartesian product of the units of the underlying sets::

        sage: C.one()
        ('', 1, 1)

    Those are implemented in the nested class
    :class:`Monoids.CartesianProducts
    <sage.categories.monoids.Monoids.CartesianProducts>` of
    ``Monoids(QQ)``. This nested class is itself a subclass of
    :class:`CartesianProductsCategory`.

    """
    _functor_name = "cartesian_product"
    _functor_category = "CartesianProducts"
    symbol = " (+) "

    def __init__(self, category=None):
        r"""
        Constructor. See :class:`CartesianProductFunctor` for details.

        TESTS::

            sage: from sage.categories.cartesian_product import CartesianProductFunctor
            sage: CartesianProductFunctor()
            The cartesian_product functorial construction
        """
        CovariantFunctorialConstruction.__init__(self)
        self._forced_category = category
        from sage.categories.sets_cat import Sets
        if self._forced_category is not None:
            codomain = self._forced_category
        else:
            codomain = Sets()
        MultivariateConstructionFunctor.__init__(self, Sets(), codomain)

    def __call__(self, args, **kwds):
        r"""
        Functorial construction application.

        This specializes the generic ``__call__`` from
        :class:`CovariantFunctorialConstruction` to:

        - handle the following plain Python containers as input:
          :class:`frozenset`, :class:`list`, :class:`set`,
          :class:`tuple`, and :class:`xrange` (Python3 ``range``).

        - handle the empty list of factors.

        See the examples below.

        EXAMPLES::

            sage: cartesian_product([[0,1], ('a','b','c')])
            The Cartesian product of ({0, 1}, {'a', 'b', 'c'})
            sage: _.category()
            Category of Cartesian products of finite enumerated sets

            sage: cartesian_product([set([0,1,2]), [0,1]])
            The Cartesian product of ({0, 1, 2}, {0, 1})
            sage: _.category()
            Category of Cartesian products of sets

        Check that the empty product is handled correctly:

            sage: C = cartesian_product([])
            sage: C
            The Cartesian product of ()
            sage: C.cardinality()
            1
            sage: C.an_element()
            ()
            sage: C.category()
            Category of Cartesian products of sets

        Check that Python3 ``range`` is handled correctly::

            sage: C = cartesian_product([range(2), range(2)])
            sage: list(C)
            [(0, 0), (0, 1), (1, 0), (1, 1)]
            sage: C.category()
            Category of Cartesian products of finite enumerated sets
        """
        if any(type(arg) in native_python_containers for arg in args):
            from sage.categories.sets_cat import Sets
            S = Sets()
            args = [S(a, enumerated_set=True) for a in args]
        elif not args:
            if self._forced_category is None:
                from sage.categories.sets_cat import Sets
                cat = Sets().CartesianProducts()
            else:
                cat = self._forced_category
            from sage.sets.cartesian_product import CartesianProduct
            return CartesianProduct((), cat)
        elif self._forced_category is not None:
            return super(CartesianProductFunctor, self).__call__(args, category=self._forced_category, **kwds)

        return super(CartesianProductFunctor, self).__call__(args, **kwds)

    def __eq__(self, other):
        r"""
        Comparison ignores the ``category`` parameter.

        TESTS::

            sage: from sage.categories.cartesian_product import CartesianProductFunctor
            sage: cartesian_product([ZZ, ZZ]).construction()[0] == CartesianProductFunctor()
            True
        """
        return isinstance(other, CartesianProductFunctor)

    def __ne__(self, other):
        r"""
        Comparison ignores the ``category`` parameter.

        TESTS::

            sage: from sage.categories.cartesian_product import CartesianProductFunctor
            sage: cartesian_product([ZZ, ZZ]).construction()[0] != CartesianProductFunctor()
            False
        """
        return not (self == other)

class CartesianProductsCategory(CovariantConstructionCategory):
    r"""
    An abstract base class for all ``CartesianProducts`` categories.

    TESTS::

        sage: C = Sets().CartesianProducts()
        sage: C
        Category of Cartesian products of sets
        sage: C.base_category()
        Category of sets
        sage: latex(C)
        \mathbf{CartesianProducts}(\mathbf{Sets})
    """

    _functor_category = "CartesianProducts"

    def _repr_object_names(self):
        """
        EXAMPLES::

            sage: ModulesWithBasis(QQ).CartesianProducts() # indirect doctest
            Category of Cartesian products of vector spaces with basis over Rational Field

        """
        # This method is only required for the capital `C`
        return "Cartesian products of %s"%(self.base_category()._repr_object_names())

    def CartesianProducts(self):
        """
        Return the category of (finite) Cartesian products of objects
        of ``self``.

        By associativity of Cartesian products, this is ``self`` (a Cartesian
        product of Cartesian products of `A`'s is a Cartesian product of
        `A`'s).

        EXAMPLES::

            sage: ModulesWithBasis(QQ).CartesianProducts().CartesianProducts()
            Category of Cartesian products of vector spaces with basis over Rational Field
        """
        return self

    def base_ring(self):
        """
        The base ring of a Cartesian product is the base ring of the underlying category.

        EXAMPLES::

            sage: Algebras(ZZ).CartesianProducts().base_ring()
            Integer Ring
        """
        return self.base_category().base_ring()

# Moved to avoid circular imports
lazy_import('sage.categories.sets_cat', 'cartesian_product')
"""
The Cartesian product functorial construction

See :class:`CartesianProductFunctor` for more information

EXAMPLES::

    sage: cartesian_product
    The cartesian_product functorial construction
"""




"""
The C3 algorithm, under control of a total order

Abstract
========

Python handles multiple inheritance by computing, for each class,
a linear extension of the poset of all its super classes (the Method
Resolution Order, MRO). The MRO is calculated recursively from local
information (the *ordered* list of the direct super classes), with
the so-called ``C3`` algorithm. This algorithm can fail if the local
information is not consistent; worst, there exist hierarchies of
classes with provably no consistent local information.

For large hierarchy of classes, like those derived from categories in
Sage, maintaining consistent local information by hand does not scale
and leads to unpredictable ``C3`` failures (the dreaded "could not
find a consistent method resolution order"); a maintenance nightmare.

This module implements a final solution to this problem. Namely, it
allows for building automatically the local information from the bare
class hierarchy in such a way that guarantees that the ``C3``
algorithm will never fail.

Err, but you said that this was provably impossible? Well, not if one
relaxes a bit the hypotheses; but that's not something one would want
to do by hand :-)

The problem
===========

Consider the following hierarchy of classes::

    sage: class A1(object): pass
    sage: class A2(object):
    ....:     def foo(self): return 2
    sage: class A3(object): pass
    sage: class A4(object):
    ....:     def foo(self): return 4
    sage: class A5(A2, A1):
    ....:     def foo(self): return 5
    sage: class A6(A4, A3): pass
    sage: class A7(A6, A5): pass

If ``a`` is an instance of ``A7``, then Python needs to choose which
implementation to use upon calling ``a.foo()``: that of ``A4`` or
``A5``, but obviously not that of ``A2``. In Python, like in many
other dynamic object oriented languages, this is achieved by
calculating once for all a specific linear extension of the hierarchy
of the super classes of each class, called its Method Resolution Order
(MRO)::

    sage: [cls.__name__ for cls in A7.mro()]
    ['A7', 'A6', 'A4', 'A3', 'A5', 'A2', 'A1', 'object']

Thus, in our example, the implementation in ``A4`` is chosen::

    sage: a = A7()
    sage: a.foo()
    4

Specifically, the MRO is calculated using the so-called ``C3``
algorithm which guarantees that the MRO respects not only inheritance,
but also the order in which the bases (direct super classes) are given
for each class.

However, for large hierarchies of classes with lots of multiple
inheritance, like those derived from categories in Sage, this
algorithm easily fails if the order of the bases is not chosen
consistently (here for ``A2`` w.r.t. ``A1``)::

    sage: class B6(A1,A2): pass
    sage: class B7(B6,A5): pass
    Traceback (most recent call last):
    ...
    TypeError: Cannot create a consistent method resolution
    order (MRO) for bases A1, A2

There actually exist hierarchies of classes for which ``C3`` fails
whatever order of the bases is chosen; the smallest such example,
admittedly artificial, has ten classes (see below). Still, this
highlights that this problem has to be tackled in a systematic way.

Fortunately, one can trick ``C3``, without changing the inheritance
semantic, by adding some super classes of ``A`` to the bases of
``A``. In the following example, we completely force a given MRO by
specifying *all* the super classes of ``A`` as bases::

    sage: class A7(A6, A5, A4, A3, A2, A1): pass
    sage: [cls.__name__ for cls in A7.mro()]
    ['A7', 'A6', 'A5', 'A4', 'A3', 'A2', 'A1', 'object']

Luckily this can be optimized; here it is sufficient to add a single
base to enforce the same MRO::

    sage: class A7(A6, A5, A4): pass
    sage: [cls.__name__ for cls in A7.mro()]
    ['A7', 'A6', 'A5', 'A4', 'A3', 'A2', 'A1', 'object']

A strategy to solve the problem
===============================

We should recall at this point a design decision that we took for the
hierarchy of classes derived from categories: *the semantic shall only
depend on the inheritance order*, not on the specific MRO, and in
particular not on the order of the bases (see
:ref:`On the order of super categories <category-primer-category-order>`).

If a choice needs to be made (for example for efficiency reasons),
then this should be done explicitly, on a method-by-method basis. In
practice this design goal is not yet met.

.. NOTE::

    When managing large hierarchies of classes in other contexts this
    may be too strong a design decision.

The strategy we use for hierarchies of classes derived from categories
is then:

1. To choose a global total order on the whole hierarchy of classes.
2. To control ``C3`` to get it to return MROs that follow this total order.

A basic approach for point 1., that will work for any hierarchy of
classes, is to enumerate the classes while they are constructed
(making sure that the bases of each class are enumerated before that
class), and to order the classes according to that enumeration. A more
conceptual ordering may be desirable, in particular to get
deterministic and reproducible results. In the context of Sage, this
is mostly relevant for those doctests displaying all the categories or
classes that an object inherits from.

Getting fine control on C3
==========================

This module is about point 2.

The natural approach would be to change the algorithm used by Python
to compute the MRO. However, changing Python's default algorithm just
for our needs is obviously not an option, and there is currently no
hook to customize specific classes to use a different algorithm.
Pushing the addition of such a hook into stock Python would take too
much time and effort.

Another approach would be to use the "adding bases" trick
straightforwardly, putting the list of *all* the super classes of a
class as its bases. However, this would have several drawbacks:

- It is not so elegant, in particular because it duplicates
  information: we already know through ``A5`` that ``A7`` is a
  subclass of ``A1``. This duplication could be acceptable in our
  context because the hierarchy of classes is generated automatically
  from a conceptual hierarchy (the categories) which serves as single
  point of truth for calculating the bases of each class.

- It increases the complexity of the calculation of the MRO with
  ``C3``. For example, for a linear hierachy of classes, the
  complexity goes from `O(n^2)` to `O(n^3)` which is not acceptable.

- It increases the complexity of inspecting the classes. For example,
  the current implementation of the ``dir`` command in Python has no
  cache, and its complexity is linear in the number of maximal paths
  in the class hierarchy graph as defined by the bases. For a linear
  hierarchy, this is of complexity `O(p_n)` where `p_n` is the number
  of integer partitions of `n`, which is exponential. And indeed,
  running ``dir`` for a typical class like
  ``GradedHopfAlgebrasWithBasis(QQ).parent_class`` with ``37`` super
  classes took `18` seconds with this approach.

  Granted: this mostly affects the ``dir`` command and could be blamed
  on its current implementation. With appropriate caching, it could be
  reimplemented to have a complexity roughly linear in the number of
  classes in the hierarchy. But this won't happen any time soon in a
  stock Python.

This module refines this approach to make it acceptable, if not
seamless. Given a hierarchy and a total order on this hierarchy, it
calculates for each element of the hierarchy the smallest list of
additional bases that forces ``C3`` to return the desired MRO. This is
achieved by implementing an instrumented variant of the ``C3``
algorithm (which we call *instrumented* ``C3``) that detects when
``C3`` is about to take a wrong decision and adds one base to force
the right decision. Then, running the standard ``C3`` algorithm with
the updated list of bases (which we call *controlled* ``C3``) yields
the desired MRO.

EXAMPLES:

As an experimentation and testing tool, we use a class
:class:`HierarchyElement` whose instances can be constructed from a
hierarchy described by a poset, a digraph, or more generally a
successor relation. By default, the desired MRO is sorted
decreasingly. Another total order can be specified using a sorting
key.

We consider the smallest poset describing a class hierarchy admitting
no MRO whatsoever::

    sage: P = Poset({10: [9,8,7], 9:[6,1], 8:[5,2], 7:[4,3], 6: [3,2], 5:[3,1], 4: [2,1] }, linear_extension=True, facade=True)

And build a :class:`HierarchyElement` from it::

    sage: from sage.misc.c3_controlled import HierarchyElement
    sage: x = HierarchyElement(10, P)

Here are its bases::

    sage: HierarchyElement(10, P)._bases
    [9, 8, 7]

Using the standard ``C3`` algorithm fails::

    sage: x.mro_standard
    Traceback (most recent call last):
    ...
    ValueError: Cannot merge the items 3, 3, 2.

We also get a failure when we relabel `P` according to another linear
extension. For easy relabelling, we first need to set an appropriate
default linear extension for `P`::

    sage: linear_extension = list(reversed(IntegerRange(1,11)))
    sage: P = P.with_linear_extension(linear_extension)
    sage: list(P)
    [10, 9, 8, 7, 6, 5, 4, 3, 2, 1]

Now we play with a specific linear extension of `P`::

    sage: Q = P.linear_extension([10, 9, 8, 7, 6, 5, 4, 1, 2, 3]).to_poset()
    sage: Q.cover_relations()
    [[10, 9], [10, 8], [10, 7], [9, 6], [9, 3], [8, 5], [8, 2], [7, 4], [7, 1], [6, 2], [6, 1], [5, 3], [5, 1], [4, 3], [4, 2]]
    sage: x = HierarchyElement(10, Q)
    sage: x.mro_standard
    Traceback (most recent call last):
    ...
    ValueError: Cannot merge the items 2, 3, 3.

On the other hand, both the instrumented ``C3`` algorithm, and the
controlled ``C3`` algorithm give the desired MRO::

    sage: x.mro
    [10, 9, 8, 7, 6, 5, 4, 3, 2, 1]
    sage: x.mro_controlled
    [10, 9, 8, 7, 6, 5, 4, 3, 2, 1]

The above checks, and more, can be run with::

    sage: x._test_mro()

In practice, the control was achieved by adding the following bases::

    sage: x._bases
    [9, 8, 7]
    sage: x._bases_controlled
    [9, 8, 7, 6, 5]

Altogether, four bases were added for control::

    sage: sum(len(HierarchyElement(q, Q)._bases) for q in Q)
    15
    sage: sum(len(HierarchyElement(q, Q)._bases_controlled) for q in Q)
    19

This information can also be recovered with::

    sage: x.all_bases_len()
    15
    sage: x.all_bases_controlled_len()
    19

We now check that the ``C3`` algorithm fails for all linear extensions
`l` of this poset, whereas both the instrumented and controlled ``C3``
algorithms succeed; along the way, we collect some statistics::

    sage: L = P.linear_extensions()
    sage: stats = []
    sage: for l in L:
    ....:     x = HierarchyElement(10, l.to_poset())
    ....:     try: # Check that x.mro_standard always fails with a ValueError
    ....:         x.mro_standard
    ....:     except ValueError:
    ....:         pass
    ....:     else:
    ....:         assert False
    ....:     assert x.mro            == list(P)
    ....:     assert x.mro_controlled == list(P)
    ....:     assert x.all_bases_len() == 15
    ....:     stats.append(x.all_bases_controlled_len()-x.all_bases_len())

Depending on the linear extension `l` it was necessary to add between
one and five bases for control; for example, `216` linear extensions
required the addition of four bases::

    sage: sorted(Word(stats).evaluation_sparse())
    [(1, 36), (2, 108), (3, 180), (4, 216), (5, 180)]

We now consider a hierarchy of categories::

    sage: from operator import attrgetter
    sage: x = HierarchyElement(Groups(), attrcall("super_categories"), attrgetter("_cmp_key"))
    sage: x.mro
    [Category of groups, Category of monoids, Category of semigroups,
     Category of inverse unital magmas, Category of unital magmas, Category of magmas,
     Category of sets, Category of sets with partial maps, Category of objects]
    sage: x.mro_standard
    [Category of groups, Category of monoids, Category of semigroups,
     Category of inverse unital magmas, Category of unital magmas, Category of magmas,
     Category of sets, Category of sets with partial maps, Category of objects]

For a typical category, few bases, if any, need to be added to force
``C3`` to give the desired order::

    sage: C = FiniteFields()
    sage: x = HierarchyElement(C, attrcall("super_categories"), attrgetter("_cmp_key"))
    sage: x.mro == x.mro_standard
    False
    sage: x.all_bases_len()
    70
    sage: x.all_bases_controlled_len()
    74

    sage: C = GradedHopfAlgebrasWithBasis(QQ)
    sage: x = HierarchyElement(C, attrcall("super_categories"), attrgetter("_cmp_key"))
    sage: x._test_mro()
    sage: x.mro == x.mro_standard
    False
    sage: x.all_bases_len()
    114
    sage: x.all_bases_controlled_len()
    117

The following can be used to search through the Sage named categories
for any that requires the addition of some bases. The output may
change a bit when the category hierarchy is changed. As long as the
list below does not change radically, it's fine to just update this
doctest::

    sage: from sage.categories.category import category_sample
    sage: sorted([C for C in category_sample()
    ....:         if len(C._super_categories_for_classes) != len(C.super_categories())],
    ....:        key=str)
    [Category of affine weyl groups,
     Category of fields,
     Category of finite dimensional algebras with basis over Rational Field,
     Category of finite dimensional hopf algebras with basis over Rational Field,
     Category of finite enumerated permutation groups,
     Category of finite weyl groups,
     Category of number fields]

AUTHOR:

- Nicolas M. Thiery (2012-09): initial version.
"""
# ****************************************************************************
#  Copyright (C) 2012-2013  Nicolas M. Thiery <nthiery at users.sf.net>
#
#  Distributed under the terms of the GNU General Public License (GPL)
#                  https://www.gnu.org/licenses/
# *****************************************************************************

from sage.misc.classcall_metaclass import ClasscallMetaclass, typecall
from sage.misc.cachefunc import cached_function, cached_method
from sage.misc.lazy_attribute import lazy_attribute
from sage.structure.dynamic_class import dynamic_class

##############################################################################
# Implementation of the total order between categories
##############################################################################

cdef tuple atoms = ("FacadeSets",
                    "FiniteSets", "Sets.Infinite", "EnumeratedSets", "SetsWithGrading",
                    "Posets", "LatticePosets", "Crystals", "AdditiveMagmas",
                    "FiniteDimensionalModules", "GradedModules", "ModulesWithBasis",
                    "Magmas", "Semigroups", "Monoids", "PermutationGroups",
                    "MagmasAndAdditiveMagmas", "Rngs", "Domains", "HopfAlgebras")


cdef dict flags = { atom: 1 << i for i,atom in enumerate(atoms) }

cdef class CmpKey:
    r"""
    This class implements the lazy attribute ``Category._cmp_key``.

    The comparison key ``A._cmp_key`` of a category is used to define
    an (almost) total order on non-join categories by setting, for two
    categories `A` and `B`, `A<B` if ``A._cmp_key > B._cmp_key``. This
    order in turn is used to give a normal form to join's, and help
    toward having a consistent method resolution order for
    parent/element classes.

    The comparison key should satisfy the following properties:

    - If `A` is a subcategory of `B`, then `A < B` (so that
      ``A._cmp_key > B._cmp_key``). In particular,
      :class:`Objects() <Objects>` is the largest category.

    - If `A != B` and taking the join of `A` and `B` makes sense
      (e.g. taking the join of ``Algebras(GF(5))`` and
      ``Algebras(QQ)`` does not make sense), then `A<B` or `B<A`.

    The rationale for the inversion above between `A<B` and
    ``A._cmp_key > B._cmp_key`` is that we want the order to
    be compatible with inclusion of categories, yet it's easier in
    practice to create keys that get bigger and bigger while we go
    down the category hierarchy.

    This implementation applies to join-irreducible categories
    (i.e. categories that are not join categories). It returns a
    pair of integers ``(flags, i)``, where ``flags`` is to be
    interpreted as a bit vector. The first bit is set if ``self``
    is a facade set. The second bit is set if ``self`` is finite.
    And so on. The choice of the flags is adhoc and was primarily
    crafted so that the order between categories would not change
    too much upon integration of :trac:`13589` and would be
    reasonably session independent. The number ``i`` is there
    to resolve ambiguities; it is session dependent, and is
    assigned increasingly when new categories are created.

    .. NOTE::

        This is currently not implemented using a
        :class:`lazy_attribute` for speed reasons only (the code is in
        Cython and takes advantage of the fact that Category objects
        always have a ``__dict__`` dictionary)

    .. TODO::

        - Handle nicely (covariant) functorial constructions and axioms

    EXAMPLES::

        sage: Objects()._cmp_key
        (0, 0)
        sage: SetsWithPartialMaps()._cmp_key
        (0, 1)
        sage: Sets()._cmp_key
        (0, 2)
        sage: Sets().Facade()._cmp_key
        (1, ...)
        sage: Sets().Finite()._cmp_key
        (2, ...)
        sage: Sets().Infinite()._cmp_key
        (4, ...)
        sage: EnumeratedSets()._cmp_key
        (8, ...)
        sage: FiniteEnumeratedSets()._cmp_key
        (10, ...)
        sage: SetsWithGrading()._cmp_key
        (16, ...)
        sage: Posets()._cmp_key
        (32, ...)
        sage: LatticePosets()._cmp_key
        (96, ...)
        sage: Crystals()._cmp_key
        (136, ...)
        sage: AdditiveMagmas()._cmp_key
        (256, ...)
        sage: Magmas()._cmp_key
        (4096, ...)
        sage: CommutativeAdditiveSemigroups()._cmp_key
        (256, ...)
        sage: Rings()._cmp_key
        (225536, ...)
        sage: Algebras(QQ)._cmp_key
        (225536, ...)
        sage: AlgebrasWithBasis(QQ)._cmp_key
        (227584, ...)
        sage: GradedAlgebras(QQ)._cmp_key
        (226560, ...)
        sage: GradedAlgebrasWithBasis(QQ)._cmp_key
        (228608, ...)

    For backward compatibility we currently want the following comparisons::

        sage: EnumeratedSets()._cmp_key > Sets().Facade()._cmp_key
        True
        sage: AdditiveMagmas()._cmp_key > EnumeratedSets()._cmp_key
        True

        sage: Category.join([EnumeratedSets(), Sets().Facade()]).parent_class._an_element_.__module__
        'sage.categories.enumerated_sets'

        sage: CommutativeAdditiveSemigroups()._cmp_key < Magmas()._cmp_key
        True
        sage: VectorSpaces(QQ)._cmp_key < Rings()._cmp_key
        True
        sage: VectorSpaces(QQ)._cmp_key < Magmas()._cmp_key
        True
    """
    cdef int count
    def __init__(self):
        """
        Sets the internal category counter to zero.

        EXAMPLES::

            sage: Objects()._cmp_key    # indirect doctest
            (0, 0)
        """
        self.count = -1

    def __get__(self, object inst, object cls):
        """
        Bind the comparison key to the given instance

        EXAMPLES::

            sage: C = Algebras(FractionField(QQ['x']))
            sage: C._cmp_key
            (225536, ...)
            sage: '_cmp_key' in C.__dict__    # indirect doctest
            True
        """
        # assert not isinstance(inst, JoinCategory)
        # Note that cls is a DynamicClassMetaclass, hence not a type
        cdef str classname = cls.__base__.__name__
        cdef int flag = flags.get(classname, 0)
        cdef object cat
        for cat in inst._super_categories:
            flag = flag | <int>(<tuple>(cat._cmp_key)[0])
        self.count += 1
        inst._cmp_key = (flag, self.count)
        return flag, self.count

_cmp_key = CmpKey()


cdef class CmpKeyNamed:
    """
    This class implements the lazy attribute ``CategoryWithParameters._cmp_key``.

    .. SEEALSO::

        - :class:`CmpKey`
        - :class:`lazy_attribute`
        - :class:`sage.categories.category.CategoryWithParameters`.

    .. NOTE::

        - The value of the attribute depends only on the parameters of
          this category.

        - This is currently not implemented using a
          :class:`lazy_attribute` for speed reasons only.

    EXAMPLES::

        sage: Algebras(GF(3))._cmp_key == Algebras(GF(5))._cmp_key  # indirect doctest
        True
        sage: Algebras(ZZ)._cmp_key != Algebras(GF(5))._cmp_key
        True

    """
    def __get__(self, object inst, object cls):
        """
        EXAMPLES::

            sage: Algebras(GF(3))._cmp_key == Algebras(GF(5))._cmp_key  # indirect doctest
            True
            sage: Algebras(ZZ)._cmp_key != Algebras(GF(5))._cmp_key
            True

        """
        cdef dict D = cls._make_named_class_cache
        cdef str name = "_cmp_key"
        cdef tuple key = (cls.__base__, name, inst._make_named_class_key(name))
        try:
            result = D[key]
            inst._cmp_key = result
            return result
        except KeyError:
            pass
        result = _cmp_key.__get__(inst,cls)
        D[key] = result
        return result

_cmp_key_named = CmpKeyNamed()

##############################################################################

def C3_merge(list lists):
    r"""
    Return the input lists merged using the ``C3`` algorithm.

    EXAMPLES::

        sage: from sage.misc.c3_controlled import C3_merge
        sage: C3_merge([[3,2],[4,3,1]])
        [4, 3, 2, 1]
        sage: C3_merge([[3,2],[4,1]])
        [3, 2, 4, 1]

    This function is only used for testing and experimenting purposes,
    but exercised quite some by the other doctests in this file.

    It is an extract of :func:`sage.misc.c3.C3_algorithm`; the latter
    could be possibly rewritten to use this one to avoid duplication.
    """
    cdef list out = []
    # Data structure / invariants:
    # We will be working with the MROs of the super objects
    # together with the list of bases of ``self``.
    # Each list is split between its head (in ``heads``) and tail (in
    # ``tails'') . Each tail is stored reversed, so that we can use a
    # cheap pop() in lieue of pop(0). A duplicate of the tail is
    # stored as a set in ``tailsets`` for cheap membership testing.
    # Since we actually want comparison by identity, not equality,
    # what we store is the set of memory locations of objects
    cdef object O, X
    cdef list tail, l
    cdef set tailset

    cdef list tails    = [l[::-1]              for l in lists if l]
    cdef list heads    = [tail.pop()             for tail in tails]
    cdef list tailsets = [set(O for O in tail) for tail in tails] # <size_t><void *>

    cdef int i, j, nbheads
    nbheads = len(heads)
    cdef bint next_item_found

    while nbheads:
        for i in range(nbheads): # from 0 <= i < nbheads:
            O = heads[i]
            # Does O appear in none of the tails?  ``all(O not in tail for tail in tailsets)``
            next_item_found = True
            for j in range(nbheads): #from 0 <= j < nbheads:
                if j == i:
                    continue
                tailset = tailsets[j]
                if O in tailset: # <size_t><void *>O
                    next_item_found = False
                    break
            if next_item_found:
                out.append(O)
                # Clear O from other heads, removing the line altogether
                # if the tail is already empty.
                # j goes down so that ``del heads[j]`` does not screw up the numbering
                for j in range(nbheads-1, -1, -1): # from nbheads > j >= 0:
                    if heads[j] == O: # is O
                        tail = tails[j]
                        if tail:
                            X = tail.pop()
                            heads[j] = X
                            tailset = tailsets[j]
                            tailset.remove(X) # <size_t><void *>X)
                        else:
                            del heads[j]
                            del tails[j]
                            del tailsets[j]
                            nbheads -= 1
                break
        if not next_item_found:
            # No head is available
            raise ValueError("Cannot merge the items %s."%', '.join(repr(head) for head in heads))
    return out

cpdef identity(x):
    r"""
    EXAMPLES::

        sage: from sage.misc.c3_controlled import identity
        sage: identity(10)
        10
    """
    return x

cpdef tuple C3_sorted_merge(list lists, key=identity):
    r"""
    Return the sorted input lists merged using the ``C3`` algorithm, with a twist.

    INPUT:

    - ``lists`` -- a non empty list (or iterable) of lists (or
      iterables), each sorted strictly decreasingly according
      to ``key``
    - ``key`` -- a function

    OUTPUT: a pair ``(result, suggestion)``

    ``result`` is the sorted list obtained by merging the lists in
    ``lists`` while removing duplicates, and ``suggestion`` is a list
    such that applying ``C3`` on ``lists`` with its last list replaced
    by ``suggestion`` would return ``result``.

    EXAMPLES:

    With the following input, :func:`C3_merge` returns right away a
    sorted list::

        sage: from sage.misc.c3_controlled import C3_merge
        sage: C3_merge([[2],[1]])
        [2, 1]

    In that case, :func:`C3_sorted_merge` returns the same result,
    with the last line unchanged::

        sage: from sage.misc.c3_controlled import C3_sorted_merge
        sage: C3_sorted_merge([[2],[1]])
        ([2, 1], [1])

    On the other hand, with the following input, :func:`C3_merge`
    returns a non sorted list::

        sage: C3_merge([[1],[2]])
        [1, 2]

    Then, :func:`C3_sorted_merge` returns a sorted list, and suggests
    to replace the last line by ``[2,1]``::

        sage: C3_sorted_merge([[1],[2]])
        ([2, 1], [2, 1])

    And indeed :func:`C3_merge` now returns the desired result::

        sage: C3_merge([[1],[2,1]])
        [2, 1]

    From now on, we use this little wrapper that checks that
    :func:`C3_merge`, with the suggestion of :func:`C3_sorted_merge`,
    returns a sorted list::

        sage: def C3_sorted_merge_check(lists):
        ....:     result, suggestion = C3_sorted_merge(lists)
        ....:     assert result == C3_merge(lists[:-1] + [suggestion])
        ....:     return result, suggestion

    Base cases::

        sage: C3_sorted_merge_check([])
        Traceback (most recent call last):
        ...
        ValueError: The input should be a non empty list of lists (or iterables)
        sage: C3_sorted_merge_check([[]])
        ([], [])
        sage: C3_sorted_merge_check([[1]])
        ([1], [1])
        sage: C3_sorted_merge_check([[3,2,1]])
        ([3, 2, 1], [3, 2, 1])
        sage: C3_sorted_merge_check([[1],[1]])
        ([1], [1])
        sage: C3_sorted_merge_check([[3,2,1],[3,2,1]])
        ([3, 2, 1], [3, 2, 1])

    Exercise different states for the last line::

        sage: C3_sorted_merge_check([[1],[2],[]])
        ([2, 1], [2, 1])
        sage: C3_sorted_merge_check([[1],[2], [1]])
        ([2, 1], [2, 1])

    Explore (all?) the different execution branches::

        sage: C3_sorted_merge_check([[3,1],[4,2]])
        ([4, 3, 2, 1], [4, 3, 2, 1])
        sage: C3_sorted_merge_check([[4,1],[3,2]])
        ([4, 3, 2, 1], [3, 2, 1])
        sage: C3_sorted_merge_check([[3,2],[4,1]])
        ([4, 3, 2, 1], [4, 3, 1])
        sage: C3_sorted_merge_check([[1],[4,3,2]])
        ([4, 3, 2, 1], [4, 3, 2, 1])
        sage: C3_sorted_merge_check([[1],[3,2], []])
        ([3, 2, 1], [2, 1])
        sage: C3_sorted_merge_check([[1],[4,3,2], []])
        ([4, 3, 2, 1], [2, 1])
        sage: C3_sorted_merge_check([[1],[4,3,2], [2]])
        ([4, 3, 2, 1], [2, 1])
        sage: C3_sorted_merge_check([[2],[1],[4],[3]])
        ([4, 3, 2, 1], [3, 2, 1])
        sage: C3_sorted_merge_check([[2],[1],[4],[]])
        ([4, 2, 1], [4, 2, 1])
        sage: C3_sorted_merge_check([[2],[1],[3],[4]])
        ([4, 3, 2, 1], [4, 3, 2, 1])
        sage: C3_sorted_merge_check([[2],[1],[3,2,1],[3]])
        ([3, 2, 1], [3])
        sage: C3_sorted_merge_check([[2],[1],[2,1],[3]])
        ([3, 2, 1], [3, 2])

    Exercises adding one item when the last list has a single element;
    the second example comes from an actual poset::

        sage: C3_sorted_merge_check([[5,4,2],[4,3],[5,4,1]])
        ([5, 4, 3, 2, 1], [5, 4, 3, 2, 1])
        sage: C3_sorted_merge_check([[6,4,2],[5,3],[6,5,1]])
        ([6, 5, 4, 3, 2, 1], [6, 5, 4, 3, 2, 1])
    """
    lists = list(lists)
    if not lists:
        raise ValueError("The input should be a non empty list of lists (or iterables)")
    #for l in lists:
    #    assert sorted(l, key = key, reverse=True) == l,\
    #        "Each input list should be sorted %s"%l

    cdef set suggestion = set(lists[-1])
    cdef bint last_list_non_empty = bool(lists[-1])
    cdef list out = []
    # Data structure / invariants:
    # - Each list remains sorted and duplicate free.
    # - Each list only evolves by popping its largest element
    #   Exception: elements can be inserted back into the last list.
    # - Whenever a list becomes empty, it's removed from the data structure.
    #   The order between the (remaining non empty) lists remains unchanged.
    # - nbheads contains the number of lists appearing in the data structure.
    # - The flag ``last_list_non_empty`` states whether the last
    #   list is currently non empty; if yes, by the above, this list is stored last.
    # - Each list is split between its head (in ``heads``) and tail (in ``tails'').
    # - Each tail is stored reversed, so that we can use a cheap ``pop()``
    #   in lieue of ``pop(0)``.
    # - A duplicate of this tail is stored as a set (of keys) in
    #   ``tailsets``, for cheap membership testing.

    cdef int i, j, max_i
    cdef bint cont
    cdef list tail, l
    cdef set tailset

    cdef list tails    = [l[::-1]                 for l in lists if l]
    cdef list heads    = [tail.pop()                for tail in tails]
    cdef set tmp_set
    cdef list tailsets = [] # remove closure [set(key(O) for O in tail) for tail in tails]
    for tail in tails:
        tmp_set = set()
        for O in tail:
            tmp_set.add(key(O))
        tailsets.append(tmp_set)
    # for i in range(len(tails)):
    #     assert len(tails[i]) == len(tailsets[i]), \
    #         "All objects should be distinct and have distinct sorting key!"+'\n'.join(" - %s: %s"%(key(O), O) for O in sorted(tails[i], key=key))

    cdef int nbheads = len(heads)
    cdef dict holder = {}

    # def print_state():
    #     print("-- %s -- %s ------"%(out,suggestion))
    #     for i in range(nbheads):
    #         print([heads[i]] + list(reversed(tails[i])))

    # def check_state():
    #     for i in range(nbheads):
    #         l = tails[i]
    #         if heads[i] is not None:
    #             l = l+[heads[i]]
    #         assert sorted(l, key=key) == l
    #         assert len(set(l)) == len(l)
    #         assert len(tails[i]) == len(set(tails[i])), \
    #             "C3's input list should have no repeats %s"%tails[i]
    #         assert set(key(O) for O in tails[i]) == tailsets[i], \
    #             "inconsistent tails[i] and tailsets[i]: %s %s"%(tails[i], tailsets[i])
    #         assert len(tails[i]) == len(tailsets[i]), \
    #             "keys should be distinct"%(tails[i])

    while nbheads:
        #print_state()
        #check_state()
        # Find the position of the largest head which will become the next item
        max_i   = 0
        max_key = key(heads[0])
        for i in range(1, nbheads): #from 1 <= i < nbheads:
            O = heads[i]
            O_key = key(O)
            if O_key > max_key:
                max_i = i
                max_key = O_key
        max_value = heads[max_i]

        # Find all the bad choices
        max_bad = None
        for i in range(max_i): #from 0 <= i < max_i:
            O = heads[i]
            # Does O appear in none of the tails?
            O_key = key(O)
            # replace the closure
            # if any(O_key in tailsets[j] for j in range(nbheads) if j != i): continue
            cont = False
            for j from 0<=j<i:
                if O_key in tailsets[j]:
                    cont = True
                    break
            if cont: continue
            for j from i<j<nbheads:
                if O_key in tailsets[j]:
                    cont = True
                    break
            if cont: continue

            # The plain C3 algorithm would have chosen O as next item!
            if max_bad is None or O_key > key(max_bad):
                max_bad = O

            # We prevent this choice by inserting O in the tail of the
            # suggestions. At this stage, we only insert it into the
            # last list. Later, we will make sure that it is actually
            # in the tail of the last list.
            if not last_list_non_empty:
                # Reinstate the last list for the suggestion
                # if it had disappeared before
                heads.append(O)
                tails.append([])
                tailsets.append(set())
                nbheads += 1
                last_list_non_empty = True
            elif O_key > key(heads[-1]):
                tails[-1].append(heads[-1])
                tailsets[-1].add(key(heads[-1]))
                heads[-1] = O
            elif O != heads[-1]:
                assert O_key not in tailsets[-1], "C3 should not have choosen this O"
                # Use a heap or something for fast sorted insertion?
                # Since Python uses TimSort, that's probably not so bad.
                tails[-1].append(O)
                tails[-1].sort(key = key)
                tailsets[-1].add(O_key)
            suggestion.add(O)
            #check_state()

        # Insert max_value in the last list, if needed to hold off the bad items
        if max_bad is not None:
            last_head = heads[-1]
            if last_head is None or key(max_bad) >= key(last_head):
                if last_head is not None and last_head != max_bad:
                    tails[-1].append(last_head)
                    tailsets[-1].add(key(last_head))
                    #check_state()
                heads[-1] = max_value
                holder[max_bad] = max_value
                #check_state()

        out.append(max_value)
        # Clear O from other heads, removing the line altogether
        # if the tail is already empty.
        # j goes down so that ``del heads[j]`` does not screw up the numbering
        for j in range(nbheads-1, -1, -1):#from nbheads > j >= 0:
            if heads[j] == max_value:
                tail = tails[j]
                if tail:
                    X = tail.pop()
                    heads[j] = X
                    tailset = tailsets[j]
                    tailset.remove(key(X))
                else:
                    del heads[j]
                    del tails[j]
                    del tailsets[j]
                    nbheads -= 1
                    if last_list_non_empty and j == nbheads:
                        last_list_non_empty = False
                #check_state()
    suggestion.update(holder.values())
    cdef list suggestion_list = sorted(suggestion, key = key, reverse=True)
    #assert C3_merge(lists[:-1]+[suggestion_list]) == out
    return (out, suggestion_list)

class HierarchyElement(object, metaclass=ClasscallMetaclass):
    """
    A class for elements in a hierarchy.

    This class is for testing and experimenting with various variants
    of the ``C3`` algorithm to compute a linear extension of the
    elements above an element in a hierarchy. Given the topic at hand,
    we use the following naming conventions. For `x` an element of the
    hierarchy, we call the elements just above `x` its *bases*, and
    the linear extension of all elements above `x` its *MRO*.

    By convention, the bases are given as lists of instances of
    :class:`HierarchyElement`, and MROs are given a list of the
    corresponding values.

    INPUT:

    - ``value`` -- an object
    - ``succ`` -- a successor function, poset or digraph from which
      one can recover the successors of ``value``
    - ``key`` -- a function taking values as input (default: the
      identity) this function is used to compute comparison keys for
      sorting elements of the hierarchy.

    .. NOTE::

        Constructing a :class:`HierarchyElement` immediately constructs the
        whole hierarchy above it.

    EXAMPLES:

    See the introduction of this module :mod:`sage.misc.c3_controlled`
    for many examples. Here we consider a large example, originaly
    taken from the hierarchy of categories above
    :class:`HopfAlgebrasWithBasis`::

        sage: from sage.misc.c3_controlled import HierarchyElement
        sage: G = DiGraph({
        ....:     44 :  [43, 42, 41, 40, 39, 38, 37, 36, 35, 34, 33, 32, 31, 30, 29, 28, 27, 26, 25, 24, 23, 22, 21, 20, 19, 18, 17, 16, 15, 14, 13, 12, 11, 10, 9, 8, 7, 6, 5, 4, 3, 2, 1, 0],
        ....:     43 :  [42, 41, 40, 36, 35, 39, 38, 37, 33, 32, 31, 30, 29, 28, 27, 26, 23, 22, 21, 20, 19, 18, 17, 16, 15, 14, 13, 12, 11, 10, 9, 8, 7, 6, 5, 4, 3, 2, 1, 0],
        ....:     42 :  [36, 35, 37, 30, 29, 28, 27, 26, 15, 14, 12, 11, 9, 8, 5, 3, 2, 1, 0],
        ....:     41 :  [40, 36, 35, 33, 32, 31, 30, 29, 28, 27, 26, 23, 22, 21, 20, 19, 18, 17, 16, 15, 14, 13, 12, 11, 10, 9, 8, 7, 6, 5, 4, 3, 2, 1, 0],
        ....:     40 :  [36, 35, 32, 31, 30, 29, 28, 27, 26, 19, 18, 17, 16, 15, 14, 13, 12, 11, 10, 9, 8, 7, 6, 5, 4, 3, 2, 1, 0],
        ....:     39 :  [38, 37, 33, 32, 31, 30, 29, 28, 27, 26, 23, 22, 21, 20, 19, 18, 17, 16, 15, 14, 13, 12, 11, 10, 9, 8, 7, 6, 5, 4, 3, 2, 1, 0],
        ....:     38 :  [37, 33, 32, 31, 30, 29, 28, 27, 26, 23, 22, 21, 20, 19, 18, 17, 16, 15, 14, 13, 12, 11, 10, 9, 8, 7, 6, 5, 4, 3, 2, 1, 0],
        ....:     37 :  [30, 29, 28, 27, 26, 15, 14, 12, 11, 9, 8, 5, 3, 2, 1, 0],
        ....:     36 :  [35, 30, 29, 28, 27, 26, 15, 14, 12, 11, 9, 8, 5, 3, 2, 1, 0],
        ....:     35 :  [29, 28, 27, 26, 15, 14, 12, 11, 9, 8, 5, 3, 2, 1, 0],
        ....:     34 :  [33, 32, 31, 30, 29, 28, 27, 26, 25, 24, 23, 22, 21, 20, 19, 18, 17, 16, 15, 14, 13, 12, 11, 10, 9, 8, 7, 6, 5, 4, 3, 2, 1, 0],
        ....:     33 :  [32, 31, 30, 29, 28, 27, 26, 23, 22, 21, 20, 19, 18, 17, 16, 15, 14, 13, 12, 11, 10, 9, 8, 7, 6, 5, 4, 3, 2, 1, 0],
        ....:     32 :  [31, 30, 29, 28, 27, 26, 19, 18, 17, 16, 15, 14, 13, 12, 11, 10, 9, 8, 7, 6, 5, 4, 3, 2, 1, 0],
        ....:     31 :  [30, 29, 28, 27, 26, 16, 15, 14, 13, 12, 11, 10, 9, 8, 7, 6, 5, 4, 3, 2, 1, 0],
        ....:     30 :  [29, 28, 27, 26, 15, 14, 12, 11, 9, 8, 5, 3, 2, 1, 0],
        ....:     29 :  [28, 27, 26, 15, 14, 12, 11, 9, 8, 5, 3, 2, 1, 0],
        ....:     28 :  [27, 26, 15, 14, 12, 11, 9, 8, 5, 3, 2, 1, 0],
        ....:     27 :  [15, 14, 12, 11, 9, 8, 5, 3, 2, 1, 0],
        ....:     26 :  [15, 14, 12, 11, 9, 8, 5, 3, 2, 1, 0],
        ....:     25 :  [24, 23, 22, 21, 20, 19, 18, 17, 16, 15, 14, 13, 12, 11, 10, 9, 8, 7, 6, 5, 4, 3, 2, 1, 0],
        ....:     24 :  [4, 2, 1, 0],
        ....:     23 :  [22, 21, 20, 19, 18, 17, 16, 15, 14, 13, 12, 11, 10, 9, 8, 7, 6, 5, 4, 3, 2, 1, 0],
        ....:     22 :  [21, 20, 18, 17, 13, 12, 11, 10, 9, 8, 7, 6, 5, 4, 3, 2, 1, 0],
        ....:     21 :  [20, 17, 4, 2, 1, 0],
        ....:     20 :  [4, 2, 1, 0],
        ....:     19 :  [18, 17, 16, 15, 14, 13, 12, 11, 10, 9, 8, 7, 6, 5, 4, 3, 2, 1, 0],
        ....:     18 :  [17, 13, 12, 11, 10, 9, 8, 7, 6, 5, 4, 3, 2, 1, 0],
        ....:     17 :  [4, 2, 1, 0],
        ....:     16 :  [15, 14, 13, 12, 11, 10, 9, 8, 7, 6, 5, 4, 3, 2, 1, 0],
        ....:     15 :  [14, 12, 11, 9, 8, 5, 3, 2, 1, 0],
        ....:     14 :  [11, 3, 2, 1, 0],
        ....:     13 :  [12, 11, 10, 9, 8, 7, 6, 5, 4, 3, 2, 1, 0],
        ....:     12 :  [11, 9, 8, 5, 3, 2, 1, 0],
        ....:     11 :  [3, 2, 1, 0],
        ....:     10 :  [9, 8, 7, 6, 5, 4, 3, 2, 1, 0],
        ....:     9 :  [8, 5, 3, 2, 1, 0],
        ....:     8 :  [3, 2, 1, 0],
        ....:     7 :  [6, 5, 4, 3, 2, 1, 0],
        ....:     6 :  [4, 3, 2, 1, 0],
        ....:     5 :  [3, 2, 1, 0],
        ....:     4 :  [2, 1, 0],
        ....:     3 :  [2, 1, 0],
        ....:     2 :  [1, 0],
        ....:     1 :  [0],
        ....:     0 :  [],
        ....:     })

        sage: x = HierarchyElement(44, G)
        sage: x.mro
        [44, 43, 42, 41, 40, 39, 38, 37, 36, 35, 34, 33, 32, 31, 30, 29, 28, 27, 26, 25, 24, 23, 22, 21, 20, 19, 18, 17, 16, 15, 14, 13, 12, 11, 10, 9, 8, 7, 6, 5, 4, 3, 2, 1, 0]
        sage: x.cls
        <class '44.cls'>
        sage: x.cls.mro()
        [<class '44.cls'>, <class '43.cls'>, <class '42.cls'>, <class '41.cls'>, <class '40.cls'>, <class '39.cls'>, <class '38.cls'>, <class '37.cls'>, <class '36.cls'>, <class '35.cls'>, <class '34.cls'>, <class '33.cls'>, <class '32.cls'>, <class '31.cls'>, <class '30.cls'>, <class '29.cls'>, <class '28.cls'>, <class '27.cls'>, <class '26.cls'>, <class '25.cls'>, <class '24.cls'>, <class '23.cls'>, <class '22.cls'>, <class '21.cls'>, <class '20.cls'>, <class '19.cls'>, <class '18.cls'>, <class '17.cls'>, <class '16.cls'>, <class '15.cls'>, <class '14.cls'>, <class '13.cls'>, <class '12.cls'>, <class '11.cls'>, <class '10.cls'>, <class '9.cls'>, <class '8.cls'>, <class '7.cls'>, <class '6.cls'>, <class '5.cls'>, <class '4.cls'>, <class '3.cls'>, <class '2.cls'>, <class '1.cls'>, <class '0.cls'>, <... 'object'>]
    """
    @staticmethod
    def __classcall__(cls, value, succ, key = None):
        """
        EXAMPLES::

            sage: from sage.misc.c3_controlled import HierarchyElement
            sage: P = Poset((divisors(30), lambda x,y: y.divides(x)), facade=True)
            sage: x = HierarchyElement(10, P)
            sage: x
            10
            sage: x.bases
            [5, 2]
            sage: x.mro
            [10, 5, 2, 1]
        """
        from sage.categories.sets_cat import Sets
        from sage.combinat.posets.poset_examples import Posets
        from sage.graphs.digraph import DiGraph
        if succ in Posets():
            assert succ in Sets().Facade()
            succ = succ.upper_covers
        if isinstance(succ, DiGraph):
            succ = succ.copy()
            succ._immutable = True
            succ = succ.neighbors_out
        if key is None:
            key = identity
        @cached_function
        def f(x):
            return typecall(cls, x, [f(y) for y in succ(x)], key, f)
        return f(value)

    def __init__(self, value, bases, key, from_value):
        """
        EXAMPLES::

            sage: from sage.misc.c3_controlled import HierarchyElement
            sage: P = Poset((divisors(30), lambda x,y: y.divides(x)), facade=True)
            sage: x = HierarchyElement(10, P)
            sage: x
            10
            sage: x.value
            10
            sage: x._bases
            [5, 2]
            sage: x._key
            <built-in function identity>
            sage: x._key(10)
            10

        The ``_from_value`` attribute is a function that can be used
        to reconstruct an element of the hierarchy from its value::

            sage: x._from_value
            Cached version of <cyfunction HierarchyElement.__classcall__.<locals>.f at ...>
            sage: x._from_value(x.value) is x
            True
        """
        self.value = value
        self._bases = sorted(bases, key=lambda x: key(x.value), reverse=True)
        self._key = key
        self._from_value = from_value

    def __repr__(self):
        """
        Return the representation of ``self`` which is that of its value.

        EXAMPLES::

            sage: from sage.misc.c3_controlled import HierarchyElement
            sage: P = Poset((divisors(30), lambda x,y: y.divides(x)), facade=True)
            sage: x = HierarchyElement(10, P)
            sage: x
            10
        """
        return repr(self.value)

    @lazy_attribute
    def bases(self):
        """
        The bases of ``self``.

        The bases are given as a list of instances of
        :class:`HierarchyElement`, sorted decreasingly according to
        the ``key`` function.

        EXAMPLES::

            sage: from sage.misc.c3_controlled import HierarchyElement
            sage: P = Poset((divisors(30), lambda x,y: y.divides(x)), facade=True)
            sage: x = HierarchyElement(10, P)
            sage: x.bases
            [5, 2]
            sage: type(x.bases[0])
            <class 'sage.misc.c3_controlled.HierarchyElement'>
            sage: x.mro
            [10, 5, 2, 1]
            sage: x._bases_controlled
            [5, 2]
        """
        return self._bases

    @lazy_attribute
    def mro(self):
        """
        The MRO for this object, calculated with :meth:`C3_sorted_merge`.

        EXAMPLES::

            sage: from sage.misc.c3_controlled import HierarchyElement, C3_sorted_merge, identity
            sage: P = Poset({7: [5,6], 5:[1,2], 6: [3,4]}, facade = True)
            sage: x = HierarchyElement(5, P)
            sage: x.mro
            [5, 2, 1]
            sage: x = HierarchyElement(6, P)
            sage: x.mro
            [6, 4, 3]
            sage: x = HierarchyElement(7, P)
            sage: x.mro
            [7, 6, 5, 4, 3, 2, 1]

            sage: C3_sorted_merge([[6, 4, 3], [5, 2, 1], [6, 5]], identity)
            ([6, 5, 4, 3, 2, 1], [6, 5, 4])

        TESTS::

            sage: assert all(isinstance(v, Integer) for v in x.mro)
        """
        bases = self._bases
        result, suggestion = C3_sorted_merge([base.mro for base in bases]+[[base.value for base in bases]], key=self._key)
        result = [self.value] + result
        self._bases_controlled = suggestion
        return result

    @lazy_attribute
    def _bases_controlled(self):
        """
        A list of bases controlled by :meth:`C3_sorted_merge`

        This triggers the calculation of the MRO using
        :meth:`C3_sorted_merge`, which sets this attribute as a side
        effect.

        EXAMPLES::

            sage: from sage.misc.c3_controlled import HierarchyElement
            sage: P = Poset({7: [5,6], 5:[1,2], 6: [3,4]}, facade = True)
            sage: x = HierarchyElement(7, P)
            sage: x._bases
            [6, 5]
            sage: x._bases_controlled
            [6, 5, 4]
        """
        self.mro
        return self._bases_controlled

    @lazy_attribute
    def mro_standard(self):
        """
        The MRO for this object, calculated with :meth:`C3_merge`

        EXAMPLES::

            sage: from sage.misc.c3_controlled import HierarchyElement, C3_merge
            sage: P = Poset({7: [5,6], 5:[1,2], 6: [3,4]}, facade=True)
            sage: x = HierarchyElement(5, P)
            sage: x.mro_standard
            [5, 2, 1]
            sage: x = HierarchyElement(6, P)
            sage: x.mro_standard
            [6, 4, 3]
            sage: x = HierarchyElement(7, P)
            sage: x.mro_standard
            [7, 6, 4, 3, 5, 2, 1]
            sage: C3_merge([[6, 4, 3], [5, 2, 1], [6, 5]])
            [6, 4, 3, 5, 2, 1]

        TESTS::

            sage: assert all(isinstance(v, Integer) for v in x.mro_standard)
        """
        bases = self._bases
        return [self.value] + C3_merge([base.mro_standard for base in bases]+[[base.value for base in bases]])

    @lazy_attribute
    def mro_controlled(self):
        """
        The MRO for this object, calculated with :meth:`C3_merge`, under
        control of :func:`C3_sorted_merge`

        EXAMPLES::

            sage: from sage.misc.c3_controlled import HierarchyElement, C3_merge
            sage: P = Poset({7: [5,6], 5:[1,2], 6: [3,4]}, facade=True)
            sage: x = HierarchyElement(5, P)
            sage: x.mro_controlled
            [5, 2, 1]
            sage: x = HierarchyElement(6, P)
            sage: x.mro_controlled
            [6, 4, 3]
            sage: x = HierarchyElement(7, P)
            sage: x.mro_controlled
            [7, 6, 5, 4, 3, 2, 1]
            sage: x._bases
            [6, 5]
            sage: x._bases_controlled
            [6, 5, 4]
            sage: C3_merge([[6, 4, 3], [5, 2, 1], [6, 5]])
            [6, 4, 3, 5, 2, 1]
            sage: C3_merge([[6, 4, 3], [5, 2, 1], [6, 5, 4]])
            [6, 5, 4, 3, 2, 1]

        TESTS::

            sage: assert all(isinstance(v, Integer) for v in x.mro_controlled)
        """
        return [self.value] + C3_merge([base.mro_controlled for base in self._bases]+[self._bases_controlled])

    @cached_method
    def _test_mro(self):
        r"""
        Runs consistency tests.

        This checks in particular that the instrumented ``C3`` and
        controlled ``C3`` algorithms give, as desired, the
        decreasingly sorted list of the objects above in the
        hierarchy. For the controlled ``C3`` algorithm, this includes
        both Sage's implementation, and Python's implementation (by
        constructing an appropriate hierarchy of classes).

        It is cached because it is run recursively on the elements
        above ``self``.

        EXAMPLES::

            sage: from sage.misc.c3_controlled import HierarchyElement
            sage: P = Poset({7: [5,6], 5:[1,2], 6: [3,4]}, facade=True)
            sage: x = HierarchyElement(7, P)
            sage: x._test_mro()
        """
        for b in self._bases:
            b._test_mro()
        try:
            assert self.mro_standard[0] == self.value
        except ValueError:
            # standard C3 failed to compute a mro; that's ok
            pass
        assert self.mro[0] == self.value
        assert self.mro_controlled[0] == self.value
        assert sorted([x.value for x in self.all_bases()], key=self._key, reverse = True) == self.mro
        assert self.mro == self.mro_controlled
        assert self.cls.mro() == [self._from_value(b).cls for b in self.mro]+[object]

    @lazy_attribute
    def cls(self):
        """
        Return a Python class with inheritance graph parallel to the hierarchy above ``self``.

        EXAMPLES::

            sage: from sage.misc.c3_controlled import HierarchyElement
            sage: P = Poset((divisors(30), lambda x,y: y.divides(x)), facade=True)
            sage: x = HierarchyElement(1, P)
            sage: x.cls
            <class '1.cls'>
            sage: x.cls.mro()
            [<class '1.cls'>, <... 'object'>]
            sage: x = HierarchyElement(30, P)
            sage: x.cls
            <class '30.cls'>
            sage: x.cls.mro()
            [<class '30.cls'>, <class '15.cls'>, <class '10.cls'>, <class '6.cls'>, <class '5.cls'>, <class '3.cls'>, <class '2.cls'>, <class '1.cls'>, <... 'object'>]
        """
        super_classes = tuple(self._from_value(base).cls for base in self._bases_controlled)
        if not super_classes:
            super_classes = (object,)
        return dynamic_class("%s.cls"%self, super_classes)


    @cached_method
    def all_bases(self):
        """
        Return the set of all instances of :class:`HierarchyElement` above
        ``self``, ``self`` included.

        EXAMPLES::

            sage: from sage.misc.c3_controlled import HierarchyElement
            sage: P = Poset((divisors(30), lambda x,y: y.divides(x)), facade=True)
            sage: HierarchyElement(1, P).all_bases()
            {1}
            sage: HierarchyElement(10, P).all_bases()  # random output
            {10, 5, 2, 1}
            sage: sorted([x.value for x in HierarchyElement(10, P).all_bases()])
            [1, 2, 5, 10]
        """
        return {self} | { x for base in self._bases for x in base.all_bases()  }

    def all_bases_len(self):
        """
        Return the cumulated size of the bases of the elements above ``self`` in the hierarchy.

        EXAMPLES::

            sage: from sage.misc.c3_controlled import HierarchyElement
            sage: P = Poset((divisors(30), lambda x,y: y.divides(x)), facade=True)
            sage: HierarchyElement(30, P).all_bases_len()
            12
        """
        return sum( len(x._bases) for x in self.all_bases())

    def all_bases_controlled_len(self):
        """
        Return the cumulated size of the controlled bases of the elements above ``self`` in the hierarchy.

        EXAMPLES::

            sage: from sage.misc.c3_controlled import HierarchyElement
            sage: P = Poset((divisors(30), lambda x,y: y.divides(x)), facade=True)
            sage: HierarchyElement(30, P).all_bases_controlled_len()
            13
        """
        return sum( len(x._bases_controlled) for x in self.all_bases())



"""
Parent objects with generators
"""

#*****************************************************************************
#       Copyright (C) 2005, 2006 William Stein <wstein@gmail.com>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 2 of the License, or
# (at your option) any later version.
#                  http://www.gnu.org/licenses/
#*****************************************************************************

from .parent_base cimport ParentWithBase


cdef class ParentWithGens(ParentWithBase):
    cdef public object _gens
    cdef public object _latex_names
    cdef public object _list



from cpython.number cimport PyNumber_TrueDivide
from sage.structure.element cimport Element


ctypedef fused ulong_or_object:
    unsigned long
    object


cpdef generic_power(a, n)
cdef generic_power_long(a, long n)
cdef generic_power_pos(a, ulong_or_object n)  # n > 0


cdef inline invert(a):
    """
    Return ``a^(-1)``.
    """
    if isinstance(a, Element):
        return ~a
    return PyNumber_TrueDivide(type(a)(1), a)


cdef inline one(a):
    """
    Return ``a^0``.
    """
    if isinstance(a, Element):
        return (<Element>a)._parent.one()
    return type(a)(1)



"""
Examples of finite semigroups
"""
#*****************************************************************************
#  Copyright (C) 2008-2009 Nicolas M. Thiery <nthiery at users.sf.net>
#
#  Distributed under the terms of the GNU General Public License (GPL)
#                  http://www.gnu.org/licenses/
#******************************************************************************

from sage.misc.cachefunc import cached_method
from sage.sets.family import Family
from sage.categories.semigroups import Semigroups
from sage.structure.parent import Parent
from sage.structure.unique_representation import UniqueRepresentation
from sage.structure.element_wrapper import ElementWrapper


class LeftRegularBand(UniqueRepresentation, Parent):
    r"""
    An example of a finite semigroup

    This class provides a minimal implementation of a finite semigroup.

    EXAMPLES::

        sage: S = FiniteSemigroups().example(); S
        An example of a finite semigroup: the left regular band generated by ('a', 'b', 'c', 'd')

    This is the semigroup generated by::

        sage: S.semigroup_generators()
        Family ('a', 'b', 'c', 'd')

    such that `x^2 = x` and `x y x = xy` for any `x` and `y` in `S`::

        sage: S('dab')
        'dab'
        sage: S('dab') * S('acb')
        'dabc'

    It follows that the elements of `S` are strings without
    repetitions over the alphabet `a`, `b`, `c`, `d`::

        sage: sorted(S.list())
        ['a', 'ab', 'abc', 'abcd', 'abd', 'abdc', 'ac', 'acb', 'acbd', 'acd',
         'acdb', 'ad', 'adb', 'adbc', 'adc', 'adcb', 'b', 'ba', 'bac',
         'bacd', 'bad', 'badc', 'bc', 'bca', 'bcad', 'bcd', 'bcda', 'bd',
         'bda', 'bdac', 'bdc', 'bdca', 'c', 'ca', 'cab', 'cabd', 'cad',
         'cadb', 'cb', 'cba', 'cbad', 'cbd', 'cbda', 'cd', 'cda', 'cdab',
         'cdb', 'cdba', 'd', 'da', 'dab', 'dabc', 'dac', 'dacb', 'db',
         'dba', 'dbac', 'dbc', 'dbca', 'dc', 'dca', 'dcab', 'dcb', 'dcba']

    It also follows that there are finitely many of them::

        sage: S.cardinality()
        64

    Indeed::

        sage: 4 * ( 1 + 3 * (1 + 2 * (1 + 1)))
        64

    As expected, all the elements of `S` are idempotents::

        sage: all( x.is_idempotent() for x in S )
        True

    Now, let us look at the structure of the semigroup::

        sage: S = FiniteSemigroups().example(alphabet = ('a','b','c'))
        sage: S.cayley_graph(side="left", simple=True).plot()
        Graphics object consisting of 60 graphics primitives
        sage: S.j_transversal_of_idempotents() # random (arbitrary choice)
        ['acb', 'ac', 'ab', 'bc', 'a', 'c', 'b']

    We conclude by running systematic tests on this semigroup::

        sage: TestSuite(S).run(verbose = True)
        running ._test_an_element() . . . pass
        running ._test_associativity() . . . pass
        running ._test_cardinality() . . . pass
        running ._test_category() . . . pass
        running ._test_construction() . . . pass
        running ._test_elements() . . .
          Running the test suite of self.an_element()
          running ._test_category() . . . pass
          running ._test_eq() . . . pass
          running ._test_new() . . . pass
          running ._test_not_implemented_methods() . . . pass
          running ._test_pickling() . . . pass
          pass
        running ._test_elements_eq_reflexive() . . . pass
        running ._test_elements_eq_symmetric() . . . pass
        running ._test_elements_eq_transitive() . . . pass
        running ._test_elements_neq() . . . pass
        running ._test_enumerated_set_contains() . . . pass
        running ._test_enumerated_set_iter_cardinality() . . . pass
        running ._test_enumerated_set_iter_list() . . . pass
        running ._test_eq() . . . pass
        running ._test_new() . . . pass
        running ._test_not_implemented_methods() . . . pass
        running ._test_pickling() . . . pass
        running ._test_some_elements() . . . pass
    """

    def __init__(self, alphabet=('a','b','c','d')):
        r"""
        A left regular band.

        EXAMPLES::

            sage: S = FiniteSemigroups().example(); S
            An example of a finite semigroup: the left regular band generated by ('a', 'b', 'c', 'd')
            sage: S = FiniteSemigroups().example(alphabet=('x','y')); S
            An example of a finite semigroup: the left regular band generated by ('x', 'y')
            sage: TestSuite(S).run()
        """
        self.alphabet = alphabet
        Parent.__init__(self, category = Semigroups().Finite().FinitelyGenerated())

    def _repr_(self):
        r"""
        TESTS::

            sage: S = FiniteSemigroups().example()
            sage: S._repr_()
            "An example of a finite semigroup: the left regular band generated by ('a', 'b', 'c', 'd')"
        """
        return "An example of a finite semigroup: the left regular band generated by %s"%(self.alphabet,)

    def product(self, x, y):
        r"""
        Returns the product of two elements of the semigroup.

        EXAMPLES::

            sage: S = FiniteSemigroups().example()
            sage: S('a') * S('b')
            'ab'
            sage: S('a') * S('b') * S('a')
            'ab'
            sage: S('a') * S('a')
            'a'

        """
        assert x in self
        assert y in self
        x = x.value
        y = y.value
        return self(x + ''.join(c for c in y if c not in x))

    @cached_method
    def semigroup_generators(self):
        r"""
        Returns the generators of the semigroup.

        EXAMPLES::

            sage: S = FiniteSemigroups().example(alphabet=('x','y'))
            sage: S.semigroup_generators()
            Family ('x', 'y')

        """
        return Family([self(i) for i in self.alphabet])

    def an_element(self):
        r"""
        Returns an element of the semigroup.

        EXAMPLES::

            sage: S = FiniteSemigroups().example()
            sage: S.an_element()
            'cdab'

            sage: S = FiniteSemigroups().example(("b"))
            sage: S.an_element()
            'b'
        """

        return self(''.join(self.alphabet[2:]+self.alphabet[0:2]))

    class Element (ElementWrapper):
        wrapped_class = str
        __lt__ = ElementWrapper._lt_by_value

Example = LeftRegularBand



r"""
Set factories
=============


A *set factory* `F` is a device for constructing some :class:`Parent`
`P` that models subsets of a big set `S`. Typically, each such parent
is constructed as the subset of `S` of all elements satisfying a
certain collection of constraints `cons`. In such a hierarchy of
subsets, one needs an easy and flexible control on how elements are
constructed. For example, one may want to construct the elements of
`P` in some subclass of the class of the elements of `S`. On other
occasions, one also often needs `P` to be a facade parent, whose
elements are represented as elements of `S` (see
:class:`~sage.categories.facade_sets.FacadeSets`).

The role of a set factory is twofold:

- *Manage a database* of constructors for the different parents `P = F(cons)`
  depending on the various kinds of constraints `cons`. Note: currently there
  is no real support for that. We are gathering use cases before fixing the
  interface.

- Ensure that the elements `e = P(...)` created by the different parents
  follows a consistent policy concerning their *class and parent*.

.. RUBRIC:: Basic usage: constructing parents through a factory

The file :mod:`sage.structure.set_factories_example` shows an example of a
:class:`SetFactory` together with typical implementation. Note that the
written code is intentionally kept minimal, many things and in particular
several iterators could be written in a more efficient way.

Consider the set `S` of couples `(x,y)` with `x` and `y` in `I:=\{0,1,2,3,4\}`.
We represent an element of `S` as a 2-elements tuple, wrapped in a class
:class:`~.set_factories_example.XYPair` deriving from :class:`ElementWrapper`.
You can create a :class:`~.set_factories_example.XYPair` with any
:class:`Parent`::

    sage: from sage.structure.set_factories import *
    sage: from sage.structure.set_factories_example import *
    sage: p = XYPair(Parent(), (0,1)); p
    (0, 1)

Now, given `(a, b)\in S` we want to consider the following subsets of
`S`

.. MATH::

    S_a := \{(x,y) \in S \mid x = a\},

    S^b := \{(x,y) \in S \mid y = b\},

    S_a^b := \{(x,y) \in S \mid x = a, y = b\}.

The constraints considered here are admittedly trivial. In a realistic
example, there would be much more of them. And for some sets of constraints
no good enumeration algorithms would be known.

In Sage, those sets are constructed by passing the constraints to the
factory. We first create the set with no constraints at all::

    sage: XYPairs
    Factory for XY pairs
    sage: S = XYPairs(); S.list()
    [(0, 0), (1, 0), ..., (4, 0), (0, 1), (1, 1), ..., (3, 4), (4, 4)]
    sage: S.cardinality()
    25

Let us construct `S_2`, `S^3` and `S_2^3`::

    sage: Sx2 = XYPairs(x=2); Sx2.list()
    [(2, 0), (2, 1), (2, 2), (2, 3), (2, 4)]
    sage: Sy3 = XYPairs(y=3); Sy3.list()
    [(0, 3), (1, 3), (2, 3), (3, 3), (4, 3)]
    sage: S23 = XYPairs(x=2, y=3); S23.list()
    [(2, 3)]

Set factories provide an alternative way to build subsets of an
already constructed set: each set constructed by a factory has a
method :meth:`~ParentWithSetFactory.subset` which accept new
constraints. Sets constructed by the factory or the
:meth:`~ParentWithSetFactory.subset` methods are identical::

    sage: Sx2s = S.subset(x=2); Sx2 is Sx2s
    True
    sage: Sx2.subset(y=3) is S23
    True

It is not possible to change an already given constraint::

    sage: S23.subset(y=5)
    Traceback (most recent call last):
    ...
    ValueError: Duplicate value for constraints 'y': was 3 now 5

.. RUBRIC:: Constructing custom elements: policies

We now come to the point of factories: constructing custom elements. The
writer of :func:`~.set_factories_example.XYPairs` decided that, by default,
the parents ``Sx2``, ``Sy3`` and ``S23`` are facade for parent ``S``. This
means that each element constructed by those subsets behaves as if they where
directly constructed by ``S`` itself::

    sage: Sx2.an_element().parent()
    AllPairs
    sage: el = Sx2.an_element()
    sage: el.parent() is S
    True
    sage: type(el) is S.element_class
    True

This is not always desirable. The device which decides how to construct an
element is called a *policy* (see :class:`SetFactoryPolicy`). Each factory
should have a default policy. Here is the policy of
:func:`~.set_factories_example.XYPairs`::

    sage: XYPairs._default_policy
    Set factory policy for <class 'sage.structure.set_factories_example.XYPair'> with parent AllPairs[=Factory for XY pairs(())]

This means that with the current policy, the parent builds elements with class
``XYPair`` and parent ``AllPairs`` which is itself constructed by calling the
factory :func:`~.set_factories_example.XYPairs` with constraints ``()``. There
is a lot of flexibility to change that. We now illustrate how to make a few
different choices.

1 - In a first use case, we want to add some methods to the constructed
elements. As illustration, we add here a new method ``sum`` which returns
`x+y`. We therefore create a new class for the elements which inherits from
:class:`~.set_factories_example.XYPair`::

    sage: class NewXYPair(XYPair):
    ....:     def sum(self):
    ....:         return sum(self.value)

Here is an instance of this class (with a dummy parent)::

    sage: el = NewXYPair(Parent(), (2,3))
    sage: el.sum()
    5

We now want to have subsets generating those new elements while still having a
single real parent (the one with no constraint) for each element. The
corresponding policy is called :class:`TopMostParentPolicy`. It takes three
parameters:

- the factory;
- the parameters for void constraint;
- the class used for elements.

Calling the factory with this policy returns a new set which builds its
elements with the new policy::

    sage: new_policy = TopMostParentPolicy(XYPairs, (), NewXYPair)
    sage: NewS = XYPairs(policy=new_policy)
    sage: el = NewS.an_element(); el
    (0, 0)
    sage: el.sum()
    0
    sage: el.parent() is NewS
    True
    sage: isinstance(el, NewXYPair)
    True

Newly constructed subsets inherit the policy::

    sage: NewS2 = NewS.subset(x=2)
    sage: el2 = NewS2.an_element(); el2
    (2, 0)
    sage: el2.sum()
    2
    sage: el2.parent() is NewS
    True

2 - In a second use case, we want the elements to remember which
parent created them. The corresponding policy is called
:class:`SelfParentPolicy`. It takes only two parameters:

- the factory;
- the class used for elements.

Here is an example::

    sage: selfpolicy = SelfParentPolicy(XYPairs, NewXYPair)
    sage: SelfS = XYPairs(policy=selfpolicy)
    sage: el = SelfS.an_element()
    sage: el.parent() is SelfS
    True

Now all subsets are the parent of the elements that they create::

    sage: SelfS2 = SelfS.subset(x=2)
    sage: el2 = SelfS2.an_element()
    sage: el2.parent() is NewS
    False
    sage: el2.parent() is SelfS2
    True

3 - Finally, a common use case is to construct simple python object which are
not Sage :class:`sage.structure.Element`. As an example, we show how to build
a parent ``TupleS`` which construct pairs as tuple. The corresponding policy
is called :class:`BareFunctionPolicy`. It takes two parameters:

- the factory;
- the function called to construct the elements.

Here is how to do it::

    sage: cons = lambda t, check: tuple(t) # ignore the check parameter
    sage: tuplepolicy = BareFunctionPolicy(XYPairs, cons)
    sage: P = XYPairs(x=2, policy=tuplepolicy)
    sage: P.list()
    [(2, 0), (2, 1), (2, 2), (2, 3), (2, 4)]
    sage: el = P.an_element()
    sage: type(el)
    <... 'tuple'>

Here are the currently implemented policies:

- :class:`FacadeParentPolicy`: reuse an existing parent together with
  its element_class

- :class:`TopMostParentPolicy`: use a parent created by the factory
  itself and provide a class ``Element`` for it. In this case, we need
  to specify the set of constraints which build this parent taking the
  ownership of all elements and the class which will be used for the
  ``Element``.

- :class:`SelfParentPolicy`: provide systematically ``Element`` and
  element_class and ensure that the parent is ``self``.

- :class:`BareFunctionPolicy`: instead of calling a class constructor element
  are passed to a function provided to the policy.

.. TODO::

    Generalize :class:`TopMostParentPolicy` to be able to have several
    topmost parents.

.. RUBRIC:: Technicalities: how policies inform parents

Parents built from factories should inherit from
:class:`ParentWithSetFactory`. This class provide a few methods
related to factories and policies. The ``__init__`` method of
:class:`ParentWithSetFactory` must be provided with the set of
constraints and the policy. A parent built from a factory must create
elements through a call to the method ``_element_constructor_``. The
current way in which policies inform parents how to builds their elements is
set by a few attributes. So the class must accept attribute
adding. The precise details of which attributes are set may be subject
to change in the future.

.. RUBRIC:: How to write a set factory

.. SEEALSO:: :mod:`.set_factories_example` for an example of a factory.

Here are the specifications:

A parent built from a factory should

- *inherit* from :class:`ParentWithSetFactory`. It should accept a
  ``policy`` argument and pass it verbatim to the ``__init__`` method
  of :class:`ParentWithSetFactory` together with the set of
  constraints;

- *create its elements* through calls to the method
  ``_element_constructor_``;

- *define a method* :class:`ParentWithSetFactory.check_element` which
  checks if a built element indeed belongs to it. The method should
  accept an extra keyword parameter called ``check`` specifying which
  level of check should be performed. It will only be called when
  ``bool(check)`` evaluates to ``True``.

The constructor of the elements of a parent from a factory should:

- receive the parent as first mandatory argument;

- accept an extra optional keyword parameter called ``check`` which is meant
  to tell if the input must be checked or not. The precise meaning of
  ``check`` is intentionally left vague. The only intent is that if
  ``bool(check)`` evaluates to ``False``, no check is performed at all.

A factory should

- *define a method* ``__call__`` which is responsible for calling the
  appropriate parent constructor given the constraints;

- *define a method* overloading :meth:`SetFactory.add_constraints` which is
  responsible of computing the union of two sets of constraints;

- *optionally* define a method or an attribute ``_default_policy`` passed to
  the :class:`ParentWithSetFactory` if no policy is given to the factory.


.. TODO::

    There is currently no support for dealing with sets of
    constraints. The set factory and the parents must cooperate to
    consistently handle them. More support, together with a generic mechanism
    to select the appropriate parent class from the constraints, will be added
    as soon as we have gathered sufficiently enough use-cases.

AUTHORS:

- Florent Hivert (2011-2012): initial revision
"""
#*****************************************************************************
#  Copyright (C) 2012 Florent Hivert <florent.hivert at lri.fr>
#
#  Distributed under the terms of the GNU General Public License (GPL)
#                  http://www.gnu.org/licenses/
#*****************************************************************************


from sage.structure.sage_object import SageObject
from sage.structure.parent import Parent
from sage.structure.unique_representation import UniqueRepresentation
from sage.misc.abstract_method import abstract_method

####################################################
#                   Factories                      #
####################################################


class SetFactory(UniqueRepresentation, SageObject):
    r"""
    This class is currently just a stub that we will be using to add
    more structures on factories.

    TESTS::

        sage: from sage.structure.set_factories import SetFactory
        sage: S = SetFactory()
        sage: S.__call__("foo")
        Traceback (most recent call last):
        ...
        NotImplementedError: <abstract method __call__ at ...>
        sage: S.add_constraints("foo")
        Traceback (most recent call last):
        ...
        NotImplementedError: <abstract method add_constraints at ...>
    """
    @abstract_method
    def __call__(self, *constraints, **consdict):
        r"""
        Construct the parent associated with the constraints in
        argument. This should return a :class:`Parent`.

        .. NOTE::

            Currently there is no specification on how constraints are
            passed as arguments.

        EXAMPLES::

            sage: from sage.structure.set_factories_example import XYPairs
            sage: XYPairs()
            AllPairs
            sage: XYPairs(3)
            {(3, b) | b in range(5)}

            sage: XYPairs(x=3)
            {(3, b) | b in range(5)}

            sage: XYPairs(y=2)
            {(a, 2) | a in range(5)}

        TESTS::

            sage: from sage.structure.set_factories import SetFactory
            sage: F = SetFactory()
            sage: F()
            Traceback (most recent call last):
            ...
            NotImplementedError: <abstract method __call__ at 0x...>
        """

    @abstract_method
    def add_constraints(self, cons, *args, **opts):
        r"""
        Add constraints to the set of constraints `cons`.

        Should return a set of constraints.

        .. NOTE::

            Currently there is no specification on how constraints are
            passed as arguments.

        EXAMPLES::

            sage: from sage.structure.set_factories_example import XYPairs
            sage: XYPairs.add_constraints((3,),((None, 2), {}))
            (3, 2)

            sage: XYPairs.add_constraints((3,),((None, None), {'y': 2}))
            (3, 2)

        TESTS::

            sage: from sage.structure.set_factories import SetFactory
            sage: F = SetFactory()
            sage: F.add_constraints(())
            Traceback (most recent call last):
            ...
            NotImplementedError: <abstract method add_constraints at 0x...>
        """

    # TODO : default policy ?

####################################################
#                    Policies                      #
####################################################


class SetFactoryPolicy(UniqueRepresentation, SageObject):
    r"""
    Abstract base class for policies.

    A policy is a device which is passed to a parent inheriting from
    :class:`ParentWithSetFactory` in order to set-up the element
    construction framework.

    INPUT:

    - ``factory`` -- a :class:`SetFactory`

    .. WARNING::

        This class is a base class for policies, one should not try
        to create instances.
    """
    def __init__(self, factory):
        r"""
        TESTS::

            sage: from sage.structure.set_factories import SetFactoryPolicy
            sage: from sage.structure.set_factories_example import XYPairs
            sage: S = SetFactoryPolicy(XYPairs); S
            <sage.structure.set_factories.SetFactoryPolicy object at ...>
        """
        assert isinstance(factory, SetFactory)
        self._factory = factory

    def factory(self):
        r"""
        Return the factory for ``self``.

        EXAMPLES::

            sage: from sage.structure.set_factories import SetFactoryPolicy, SelfParentPolicy
            sage: from sage.structure.set_factories_example import XYPairs, XYPair
            sage: XYPairs._default_policy.factory()
            Factory for XY pairs
            sage: XYPairs._default_policy.factory() is XYPairs
            True

        TESTS::

            sage: policy = SetFactoryPolicy(XYPairs)
            sage: policy.factory()
            Factory for XY pairs
            sage: SelfParentPolicy(XYPairs, XYPair).factory()
            Factory for XY pairs
        """
        return self._factory

    def self_element_constructor_attributes(self, Element):
        r"""
        Element Constructor Attributes for non facade parent.

        The list of attributes which must be set during the init of a
        non facade parent with factory.

        INPUT:

        - ``Element`` -- the class used for the elements

        EXAMPLES::

            sage: from sage.structure.set_factories_example import XYPairs, XYPair
            sage: pol = XYPairs._default_policy
            sage: pol.self_element_constructor_attributes(XYPair)
            {'Element': <class 'sage.structure.set_factories_example.XYPair'>,
             '_parent_for': 'self'}
        """
        return {'_parent_for': "self", 'Element': Element}

    def facade_element_constructor_attributes(self, parent):
        r"""
        Element Constructor Attributes for facade parent.

        The list of attributes which must be set during the init of a
        facade parent with factory.

        INPUT:

        - ``parent`` -- the actual parent for the elements

        EXAMPLES::

            sage: from sage.structure.set_factories_example import XYPairs, XYPair
            sage: pol = XYPairs._default_policy
            sage: pol.facade_element_constructor_attributes(XYPairs())
            {'_facade_for': AllPairs,
             '_parent_for': AllPairs,
             'element_class': <class 'sage.structure.set_factories_example.AllPairs_with_category.element_class'>}
        """
        return {'_parent_for': parent,
                '_facade_for': parent,
                'element_class': parent.element_class}

    @abstract_method
    def element_constructor_attributes(self, constraints):
        r"""
        Element constructor attributes.

        INPUT:

        - ``constraints`` -- a bunch of constraints

        Should return the attributes that are prerequisite for element
        construction. This is coordinated with
        :meth:`ParentWithSetFactory._element_constructor_`. Currently two standard
        attributes are provided in
        :meth:`facade_element_constructor_attributes` and
        :meth:`self_element_constructor_attributes`. You should return the
        one needed depending on the given constraints.

        EXAMPLES::

            sage: from sage.structure.set_factories_example import XYPairs, XYPair
            sage: pol = XYPairs._default_policy
            sage: pol.element_constructor_attributes(())
            {'Element': <class 'sage.structure.set_factories_example.XYPair'>,
             '_parent_for': 'self'}
            sage: pol.element_constructor_attributes((1))
            {'_facade_for': AllPairs,
             '_parent_for': AllPairs,
             'element_class': <class 'sage.structure.set_factories_example.AllPairs_with_category.element_class'>}
        """


class SelfParentPolicy(SetFactoryPolicy):
    r"""
    Policy where each parent is a standard parent.

    INPUT:

    - ``factory`` -- an instance of :class:`SetFactory`
    - ``Element`` -- a subclass of :class:`~.element.Element`

    Given a factory ``F`` and a class ``E``, returns a policy for
    parent ``P`` creating elements in class ``E`` and parent ``P``
    itself.

    EXAMPLES::

        sage: from sage.structure.set_factories import SelfParentPolicy
        sage: from sage.structure.set_factories_example import XYPairs, XYPair, Pairs_Y
        sage: pol = SelfParentPolicy(XYPairs, XYPair)
        sage: S = Pairs_Y(3, pol)
        sage: el = S.an_element()
        sage: el.parent() is S
        True

        sage: class Foo(XYPair): pass
        sage: pol = SelfParentPolicy(XYPairs, Foo)
        sage: S = Pairs_Y(3, pol)
        sage: el = S.an_element()
        sage: isinstance(el, Foo)
        True
    """
    def __init__(self, factory, Element):
        r"""
        TESTS::

            sage: from sage.structure.set_factories import SelfParentPolicy
            sage: from sage.structure.set_factories_example import XYPairs, XYPair
            sage: S = SelfParentPolicy(XYPairs, XYPair); S
            Set factory policy for <class 'sage.structure.set_factories_example.XYPair'> with parent ``self``
            sage: TestSuite(S).run(skip='_test_category')
        """
        self._Element = Element
        SetFactoryPolicy.__init__(self, factory)

    def element_constructor_attributes(self, constraints):
        r"""
        Return the element constructor attributes as per
        :meth:`SetFactoryPolicy.element_constructor_attributes`

        INPUT:

        - ``constraints`` -- a bunch of constraints

        TESTS::

            sage: from sage.structure.set_factories import SelfParentPolicy
            sage: from sage.structure.set_factories_example import XYPairs, XYPair
            sage: pol = SelfParentPolicy(XYPairs, XYPair)
            sage: pol.element_constructor_attributes(())
            {'Element': <class 'sage.structure.set_factories_example.XYPair'>,
             '_parent_for': 'self'}
        """
        return self.self_element_constructor_attributes(self._Element)

    def _repr_(self):
        r"""
        TESTS::

            sage: from sage.structure.set_factories import SelfParentPolicy
            sage: from sage.structure.set_factories_example import XYPairs, XYPair
            sage: SelfParentPolicy(XYPairs, XYPair)    # indirect doctest
            Set factory policy for <class 'sage.structure.set_factories_example.XYPair'> with parent ``self``
        """
        return "Set factory policy for {} with parent ``self``".format(self._Element)


class TopMostParentPolicy(SetFactoryPolicy):
    r"""
    Policy where the parent of the elements is the topmost parent.

    INPUT:

    - ``factory`` -- an instance of :class:`SetFactory`
    - ``top_constraints`` -- the empty set of constraints.
    - ``Element`` -- a subclass of :class:`~.element.Element`

    Given a factory ``F`` and a class ``E``, returns a policy for
    parent ``P`` creating element in class ``E`` and parent
    ``factory(*top_constraints, policy)``.

    EXAMPLES::

        sage: from sage.structure.set_factories_example import XYPairs, XYPair
        sage: P = XYPairs(); P.policy()
        Set factory policy for <class 'sage.structure.set_factories_example.XYPair'> with parent AllPairs[=Factory for XY pairs(())]
    """
    def __init__(self, factory, top_constraints, Element):
        """
        TESTS::

            sage: from sage.structure.set_factories import TopMostParentPolicy
            sage: from sage.structure.set_factories_example import XYPairs, XYPair
            sage: T = TopMostParentPolicy(XYPairs, (), XYPair); T
            Set factory policy for <class 'sage.structure.set_factories_example.XYPair'> with parent AllPairs[=Factory for XY pairs(())]
            sage: TestSuite(T).run(skip='_test_category')
        """
        # assert(isinstance(top_constraints, tuple))
        self._top_constraints = top_constraints
        self._Element = Element
        SetFactoryPolicy.__init__(self, factory)

    def element_constructor_attributes(self, constraints):
        r"""
        Return the element constructor attributes as per
        :meth:`SetFactoryPolicy.element_constructor_attributes`.

        INPUT:

        - ``constraints`` -- a bunch of constraints

        TESTS::

            sage: from sage.structure.set_factories import TopMostParentPolicy
            sage: from sage.structure.set_factories_example import XYPairs, XYPair
            sage: pol = TopMostParentPolicy(XYPairs, (), XYPair)
            sage: pol.element_constructor_attributes(())
            {'Element': <class 'sage.structure.set_factories_example.XYPair'>,
             '_parent_for': 'self'}
            sage: pol.element_constructor_attributes((1))
            {'_facade_for': AllPairs,
             '_parent_for': AllPairs,
             'element_class': <class 'sage.structure.set_factories_example.AllPairs_with_category.element_class'>}
        """
        factory = self._factory
        if constraints == self._top_constraints:
            return self.self_element_constructor_attributes(self._Element)
        else:
            return self.facade_element_constructor_attributes(
                factory(*self._top_constraints, policy=self))

    def _repr_(self):
        r"""
        TESTS::

            sage: from sage.structure.set_factories import TopMostParentPolicy
            sage: from sage.structure.set_factories_example import XYPairs, XYPair
            sage: TopMostParentPolicy(XYPairs, (), XYPair)  # indirect doctest
            Set factory policy for <class 'sage.structure.set_factories_example.XYPair'> with parent AllPairs[=Factory for XY pairs(())]
        """
        return "Set factory policy for {} with parent {}[={}({})]".format(
            self._Element, self._factory(*self._top_constraints, policy=self),
            self._factory, self._top_constraints)


class FacadeParentPolicy(SetFactoryPolicy):
    r"""
    Policy for facade parent.

    INPUT:

    - ``factory`` -- an instance of :class:`SetFactory`
    - ``parent`` -- an instance of :class:`Parent`

    Given a factory ``F`` and a class ``E``, returns a policy for
    parent ``P`` creating elements as if they were created by
    ``parent``.

    EXAMPLES::

        sage: from sage.structure.set_factories import SelfParentPolicy, FacadeParentPolicy
        sage: from sage.structure.set_factories_example import XYPairs, XYPair

    We create a custom standard parent ``P``::

        sage: selfpolicy = SelfParentPolicy(XYPairs, XYPair)
        sage: P = XYPairs(x=2, policy=selfpolicy)
        sage: pol = FacadeParentPolicy(XYPairs, P)
        sage: P2 = XYPairs(x=2, y=3, policy=pol)
        sage: el = P2.an_element()
        sage: el.parent() is P
        True
        sage: type(el) is P.element_class
        True

    If ``parent`` is itself a facade parent, then transitivity is
    correctly applied::

        sage: P =  XYPairs()
        sage: P2 = XYPairs(x=2)
        sage: P2.category()
        Category of facade finite enumerated sets
        sage: pol = FacadeParentPolicy(XYPairs, P)
        sage: P23 = XYPairs(x=2, y=3, policy=pol)
        sage: el = P2.an_element()
        sage: el.parent() is P
        True
        sage: type(el) is P.element_class
        True
    """
    def __init__(self, factory, parent):
        r"""
        TESTS::

            sage: from sage.structure.set_factories import FacadeParentPolicy
            sage: from sage.structure.set_factories_example import XYPairs, XYPair
            sage: F = FacadeParentPolicy(XYPairs, XYPairs()); F
            Set factory policy for facade parent AllPairs
            sage: TestSuite(F).run(skip='_test_category')
        """
        self._parent_for = parent
        SetFactoryPolicy.__init__(self, factory)

    def element_constructor_attributes(self, constraints):
        r"""
        Return the element constructor attributes as per
        :meth:`SetFactoryPolicy.element_constructor_attributes`.

        INPUT:

        - ``constraints`` -- a bunch of constraints

        TESTS::

            sage: from sage.structure.set_factories import FacadeParentPolicy
            sage: from sage.structure.set_factories_example import XYPairs, XYPair
            sage: pol = FacadeParentPolicy(XYPairs, XYPairs())
            sage: pol.element_constructor_attributes(())
            {'_facade_for': AllPairs,
             '_parent_for': AllPairs,
             'element_class': <class 'sage.structure.set_factories_example.AllPairs_with_category.element_class'>}
            sage: pol.element_constructor_attributes((1))
            {'_facade_for': AllPairs,
             '_parent_for': AllPairs,
             'element_class': <class 'sage.structure.set_factories_example.AllPairs_with_category.element_class'>}
        """
        return self.facade_element_constructor_attributes(
            self._parent_for._parent_for)

    def _repr_(self):
        r"""
        TESTS::

            sage: from sage.structure.set_factories import FacadeParentPolicy
            sage: from sage.structure.set_factories_example import XYPairs, XYPair
            sage: FacadeParentPolicy(XYPairs, XYPairs())  # indirect doctest
            Set factory policy for facade parent AllPairs
        """
        return "Set factory policy for facade parent {}".format(
            self._parent_for)




class BareFunctionPolicy(SetFactoryPolicy):
    r"""
    Policy where element are constructed using a bare function.

    INPUT:

    - ``factory`` -- an instance of :class:`SetFactory`
    - ``constructor`` -- a function

    Given a factory ``F`` and a function ``c``, returns a policy for
    parent ``P`` creating element using the function ``f``.

    EXAMPLES::

        sage: from sage.structure.set_factories import BareFunctionPolicy
        sage: from sage.structure.set_factories_example import XYPairs
        sage: cons = lambda t, check: tuple(t) # ignore the check parameter
        sage: tuplepolicy = BareFunctionPolicy(XYPairs, cons)
        sage: P = XYPairs(x=2, policy=tuplepolicy)
        sage: el = P.an_element()
        sage: type(el)
        <... 'tuple'>
    """
    def __init__(self, factory, constructor):
        """
        TESTS::

            sage: from sage.structure.set_factories import BareFunctionPolicy
            sage: from sage.structure.set_factories_example import XYPairs
            sage: pol = BareFunctionPolicy(XYPairs, tuple)
            sage: TestSuite(pol).run(skip='_test_category')
        """
        # assert(isinstance(top_constraints, tuple))
        self._constructor = constructor
        SetFactoryPolicy.__init__(self, factory)

    def element_constructor_attributes(self, constraints):
        r"""
        Return the element constructor attributes as per
        :meth:`SetFactoryPolicy.element_constructor_attributes`.

        INPUT:

        - ``constraints`` -- a bunch of constraints

        TESTS::

            sage: from sage.structure.set_factories import BareFunctionPolicy
            sage: from sage.structure.set_factories_example import XYPairs
            sage: pol = BareFunctionPolicy(XYPairs, tuple)
            sage: pol.element_constructor_attributes(())
            {'_element_constructor_': <... 'tuple'>, '_parent_for': None}
        """
        return {'_element_constructor_' : self._constructor, '_parent_for' : None}

    def _repr_(self):
        r"""
        TESTS::

            sage: from sage.structure.set_factories import BareFunctionPolicy
            sage: from sage.structure.set_factories_example import XYPairs
            sage: BareFunctionPolicy(XYPairs, tuple)
            Set factory policy for bare function <... 'tuple'>
        """
        return "Set factory policy for bare function {}".format(self._constructor)


####################################################
#                     Parent                       #
####################################################


class ParentWithSetFactory(Parent):
    r"""
    Abstract class for parent belonging to a set factory.

    INPUT:

    - ``constraints`` -- a set of constraints
    - ``policy`` -- the policy for element construction
    - ``category`` -- the category of the parent (default to ``None``)

    Depending on the constraints and the policy, initialize the parent
    in a proper category to set up element construction.

    EXAMPLES::

        sage: from sage.structure.set_factories_example import XYPairs, PairsX_
        sage: P = PairsX_(3, XYPairs._default_policy)
        sage: P is XYPairs(3)
        True
        sage: P.category()
        Category of facade finite enumerated sets
    """
    def __init__(self, constraints, policy, category=None):
        r"""
        TESTS::

            sage: from sage.structure.set_factories import ParentWithSetFactory
            sage: from sage.structure.set_factories_example import XYPairs
            sage: isinstance(XYPairs(3), ParentWithSetFactory)  # indirect doctest
            True
        """
        self._constraints = constraints
        assert isinstance(policy, SetFactoryPolicy)
        self._policy = policy
        policy_attributes = policy.element_constructor_attributes(constraints)
        for attrname, attr in policy_attributes.items():
            if attr == "self":
                setattr(self, attrname, self)
            else:
                setattr(self, attrname, attr)
        assert self._parent_for is None or isinstance(self._parent_for, Parent)
        Parent.__init__(self,
                        category=category,
                        facade=policy_attributes.get('_facade_for', None))

    def constraints(self):
        r"""
        Return the constraints defining ``self``.

        .. NOTE::

            Currently there is no specification on how constraints are
            passed as arguments.

        EXAMPLES::

            sage: from sage.structure.set_factories_example import XYPairs
            sage: XYPairs().constraints()
            ()
            sage: XYPairs(x=3).constraints()
            (3, None)
            sage: XYPairs(y=2).constraints()
            (None, 2)
        """
        return self._constraints

    def policy(self):
        r"""
        Return the policy used when ``self`` was created.

        EXAMPLES::

            sage: from sage.structure.set_factories_example import XYPairs
            sage: XYPairs().policy()
            Set factory policy for <class 'sage.structure.set_factories_example.XYPair'> with parent AllPairs[=Factory for XY pairs(())]
            sage: XYPairs(x=3).policy()
            Set factory policy for <class 'sage.structure.set_factories_example.XYPair'> with parent AllPairs[=Factory for XY pairs(())]
        """
        return self._policy

    def facade_policy(self):
        r"""
        Return the policy for parent facade for ``self``.

        EXAMPLES::

            sage: from sage.structure.set_factories import SelfParentPolicy
            sage: from sage.structure.set_factories_example import XYPairs, XYPair

        We create a custom standard parent ``P``::

            sage: selfpolicy = SelfParentPolicy(XYPairs, XYPair)
            sage: P = XYPairs(x=2, policy=selfpolicy)
            sage: P.facade_policy()
            Set factory policy for facade parent {(2, b) | b in range(5)}

        Now passing ``P.facade_policy()`` creates parent which are facade for
        ``P``::

            sage: P3 = XYPairs(x=2, y=3, policy=P.facade_policy())
            sage: P3.facade_for() == (P,)
            True
            sage: el = P3.an_element()
            sage: el.parent() is P
            True
        """
        return FacadeParentPolicy(self.factory(), self)

    def factory(self):
        r"""
        Return the factory which built ``self``.

        EXAMPLES::

            sage: from sage.structure.set_factories_example import XYPairs
            sage: XYPairs().factory() is XYPairs
            True
            sage: XYPairs(x=3).factory() is XYPairs
            True
        """
        return self._policy.factory()

    def subset(self, *args, **options):
        r"""
        Return a subset of ``self`` by adding more constraints.

        .. NOTE::

            Currently there is no specification on how constraints are
            passed as arguments.

        EXAMPLES::

            sage: from sage.structure.set_factories_example import XYPairs
            sage: S = XYPairs()
            sage: S3 = S.subset(x=3)
            sage: S3.list()
            [(3, 0), (3, 1), (3, 2), (3, 3), (3, 4)]

        TESTS::

            sage: S3 is XYPairs(3)
            True
            sage: S3 is XYPairs(x=3)
            True
        """
        factory = self.factory()
        constr = factory.add_constraints(self._constraints,
                                         (args, options))
        return factory(*constr, policy=self._policy)

    def _test_subset(self, **options):
        r"""
        Tests that subsets with no extra parameters returns
        ``self``.

        Currently, only the test that one gets the same parent when no
        more constraints are given, is performed.

        .. TODO::

            Straighten the test when handling of constraints will be
            specified.

        EXAMPLES::

            sage: from sage.structure.set_factories_example import XYPairs
            sage: S = XYPairs()
            sage: S._test_subset()
        """
        tester = self._tester(**options)
        tester.assertIs(self.subset(), self)

    @abstract_method
    def check_element(self, x, check):
        r"""
        Check that ``x`` verifies the constraints of ``self``.

        INPUT:

        - ``x`` -- an instance of ``self.element_class``.

        - ``check`` -- the level of checking to be performed (usually a
          boolean).

        This method may assume that ``x`` was properly constructed by
        ``self`` or a possible super-set of ``self`` for which
        ``self`` is a facade. It should return nothing if ``x``
        verifies the constraints and raise a
        :exc:`~exceptions.ValueError` explaining which constraints
        ``x`` fails otherwise.

        The method should accept an extra parameter check specifying
        which level of check should be performed. It will only be
        called when ``bool(check)`` evaluates to ``True``.

        .. TODO::

            Should we always call check element and let it decide
            which check has to be performed ?

        EXAMPLES::

            sage: from sage.structure.set_factories_example import XYPairs
            sage: S = XYPairs()
            sage: el = S((2,3))
            sage: S.check_element(el, True)
            sage: XYPairs(x=2).check_element(el, True)
            sage: XYPairs(x=3).check_element(el, True)
            Traceback (most recent call last):
            ...
            ValueError: Wrong first coordinate
            sage: XYPairs(y=4).check_element(el, True)
            Traceback (most recent call last):
            ...
            ValueError: Wrong second coordinate
        """

    def __contains__(self, x):
        r"""
        Default implementation for ``__contains__``.

        INPUT:

        - ``x`` -- any object

        Check for class, parent and calls ``self.check_element(x)``.

        TESTS::

            sage: from sage.structure.set_factories_example import XYPairs
            sage: S = XYPairs()
            sage: el = S((2,3))
            sage: el in S
            True
            sage: el in XYPairs(x=2)
            True
            sage: el in XYPairs(x=3)
            False
            sage: el in XYPairs(y=4)
            False
        """
        if (isinstance(x, self.element_class) and
            x.parent() == self._parent_for):  # TODO: is_parent_of ???
            try:
                self.check_element(x, True)
            except ValueError:
                return False
            else:
                return True
        return False

    def __call__(self, *args, **keywords):
        r"""
        TESTS::

            sage: from sage.structure.set_factories_example import XYPairs
            sage: S = XYPairs()
            sage: el = S((2,3)); el
            (2, 3)
            sage: S(el) is el
            True

            sage: XYPairs(x=3)((2,3))
            Traceback (most recent call last):
            ...
            ValueError: Wrong first coordinate

            sage: XYPairs(x=3)(el)
            Traceback (most recent call last):
            ...
            ValueError: Wrong first coordinate
        """
        # Ensure idempotence of element construction
        if (len(args) == 1 and
            isinstance(args[0], self.element_class) and
            args[0].parent() == self._parent_for):
            check = keywords.get("check", True)
            if check:
                self.check_element(args[0], check)
            return args[0]
        else:
            return Parent.__call__(self, *args, **keywords)

    # QUESTION: Should we call:
    #     self._parent_for._element_constructor_
    # currently we do not call it directly because:
    #  - it may do some extra check we dont want to perform ?
    #  - calling directly element_class should be faster
    def _element_constructor_(self, *args, **keywords):
        r"""
        TESTS::

            sage: from sage.structure.set_factories_example import XYPairs
            sage: XYPairs()((2,3)) # indirect doctest
            (2, 3)
            sage: XYPairs(x=3)((3,3)) # indirect doctest
            (3, 3)
            sage: XYPairs(x=3)((2,3)) # indirect doctest
            Traceback (most recent call last):
            ...
            ValueError: Wrong first coordinate

            sage: XYPairs(x=3)((2,3), check=False) # Don't do this at home, kids
            (2, 3)
        """
        check = keywords.get("check", True)
        res = self.element_class(self._parent_for, *args, **keywords)
        if check:
            self.check_element(res, check)
        return res
